@@ -1,19 +1,20 @@
 /*
  * The Sleuth Kit
  * 
- * $Date: 2005/09/02 19:53:26 $
+ * $Date: 2006/05/11 15:45:50 $
  */
 #ifndef _AUX_LIB_H
 #define _AUX_LIB_H
 
-#include "error.h"
 #include "mymalloc.h"
 #include "split_at.h"
 #include "tsk_types.h"
 #include "unicode.h"
 #include "tsk_endian.h"
 #include "data_buf.h"
+#include "tsk_error.h"
 
 extern void print_version(FILE *);
+extern SSIZE_T parse_offset(char *);
 
 #endif
