@@ -1,17 +1,15 @@
 /*
 ** The Sleuth Kit
 **
-** $Date: 2005/09/02 19:53:27 $
+** $Date: 2006/04/06 01:15:55 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
+** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
 **
 ** TASK
 ** Copyright (c) 2002 Brian Carrier, @stake Inc.  All rights reserved
 **
-** renamed to have consistant name (was unrm)
-**
-** 
 ** Copyright (c) 1997,1998,1999, International Business Machines
 ** Corporation and others. All Rights Reserved.
 **
@@ -31,44 +29,25 @@
 #include "libfstools.h"
 
 
-/* atoblock - convert string to block number */
-
-DADDR_T
-atoblock(const char *str)
-{
-    char *cp;
-    DADDR_T addr;
-
-    if (*str == 0)
-	return (0);
-
-    addr = strtoull(str, &cp, 0);
-    if (*cp || cp == str)
-	error("bad block number: %s", str);
-
-    return (addr);
-}
-
 /* usage - explain and terminate */
 
 static void
 usage()
 {
     fprintf(stderr,
-	    "usage: %s [-belvV] [-f fstype] [-i imgtype] [-o imgoffset] image [images] [start-stop]\n",
-	    progname);
+	"usage: %s [-belvV] [-f fstype] [-i imgtype] [-o imgoffset] image [images] [start-stop]\n",
+	progname);
     fprintf(stderr, "\t-b: no block padding\n");
     fprintf(stderr, "\t-e: every block\n");
     fprintf(stderr, "\t-l: print details in time machine list format\n");
     fprintf(stderr, "\t-i imgtype: The format of the image file\n");
     fprintf(stderr,
-	    "\t-o imgoffset: The offset of the file system in the image (in sectors)\n");
+	"\t-o imgoffset: The offset of the file system in the image (in sectors)\n");
     fprintf(stderr,
-	    "\t-s: print slack space only (other flags are ignored\n");
+	"\t-s: print slack space only (other flags are ignored\n");
     fprintf(stderr, "\t-v: verbose to stderr\n");
     fprintf(stderr, "\t-V: print version\n");
     fprintf(stderr, "\t-f fstype: File system type\n");
-    fprintf(stderr, "Supported file system types:\n");
     fs_print_types(stderr);
     fprintf(stderr, "Supported image format types:\n");
     img_print_types(stderr);
@@ -94,7 +73,8 @@ main(int argc, char **argv)
 	FS_FLAG_DATA_CONT;
     char *fstype = NULL;
     char lclflags = DLS_CAT, set_bounds = 1;
-    char *imgtype = NULL, *imgoff = NULL, *cp, *dash;
+    char *imgtype = NULL, *cp, *dash;
+    SSIZE_T imgoff = 0;
     IMG_INFO *img;
 
     progname = argv[0];
@@ -122,7 +102,10 @@ main(int argc, char **argv)
 	    lclflags = DLS_LIST;
 	    break;
 	case 'o':
-	    imgoff = optarg;
+	    if ((imgoff = parse_offset(optarg)) == -1) {
+		tsk_error_print(stderr);
+		exit(1);
+	    }
 	    break;
 
 	case 's':
@@ -149,16 +132,28 @@ main(int argc, char **argv)
 
 	if (lclflags & DLS_LIST) {
 	    fprintf(stderr,
-		    "Other options igroned with the slack space flag, try again\n");
+		"Other options igroned with the slack space flag, try again\n");
 	    exit(1);
 	}
 
 	/* There should be no other arguments */
 	img =
-	    img_open(imgtype, imgoff, argc - optind,
-		     (const char **) &argv[optind]);
+	    img_open(imgtype, argc - optind,
+	    (const char **) &argv[optind]);
+
+	if (img == NULL) {
+	    tsk_error_print(stderr);
+	    exit(1);
+	}
+
+	if ((fs = fs_open(img, imgoff, fstype)) == NULL) {
+	    tsk_error_print(stderr);
+	    if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
+		fs_print_types(stderr);
+	    img->close(img);
+	    exit(1);
+	}
 
-	fs = fs_open(img, fstype);
     }
     else {
 
@@ -166,9 +161,14 @@ main(int argc, char **argv)
 	/* We need to determine if the block range was given */
 	if ((dash = strchr(argv[argc - 1], '-')) == NULL) {
 	    /* No dash in arg - therefore it is an image file name */
-	    img =
-		img_open(imgtype, imgoff, argc - optind,
-			 (const char **) &argv[optind]);
+	    if ((img =
+		    img_open(imgtype, argc - optind,
+			(const char **) &argv[optind])) == NULL) {
+
+		tsk_error_print(stderr);
+		exit(1);
+	    }
+
 	    set_bounds = 1;
 
 	}
@@ -180,9 +180,13 @@ main(int argc, char **argv)
 	    if (*cp || cp == argv[argc - 1]) {
 		/* Not a number - consider it a file name */
 		*dash = '-';
-		img =
-		    img_open(imgtype, imgoff, argc - optind,
-			     (const char **) &argv[optind]);
+		if ((img =
+			img_open(imgtype, argc - optind,
+			    (const char **) &argv[optind])) == NULL) {
+		    tsk_error_print(stderr);
+		    exit(1);
+		}
+
 		set_bounds = 1;
 	    }
 	    else {
@@ -193,24 +197,38 @@ main(int argc, char **argv)
 		    /* Not a number - consider it a file name */
 		    dash--;
 		    *dash = '-';
-		    img =
-			img_open(imgtype, imgoff, argc - optind,
-				 (const char **) &argv[optind]);
+		    if ((img =
+			    img_open(imgtype, argc - optind,
+				(const char **) &argv[optind])) == NULL) {
+			tsk_error_print(stderr);
+			exit(1);
+		    }
+
 		    set_bounds = 1;
 		}
 		else {
 
 		    set_bounds = 0;
 		    /* It was a block range, so do not include it in the open */
-		    img =
-			img_open(imgtype, imgoff, argc - optind - 1,
-				 (const char **) &argv[optind]);
+		    if ((img =
+			    img_open(imgtype, argc - optind - 1,
+				(const char **) &argv[optind])) == NULL) {
+			tsk_error_print(stderr);
+			exit(1);
+		    }
+
 		}
 	    }
 	}
 
+	if ((fs = fs_open(img, imgoff, fstype)) == NULL) {
+	    tsk_error_print(stderr);
+	    if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
+		fs_print_types(stderr);
+	    img->close(img);
+	    exit(1);
+	}
 
-	fs = fs_open(img, fstype);
 
 	/* do we need to set the range or just check them? */
 	if (set_bounds) {
@@ -226,7 +244,13 @@ main(int argc, char **argv)
 	}
     }
 
-    fs_dls(fs, lclflags, bstart, blast, flags);
+    if (fs_dls(fs, lclflags, bstart, blast, flags)) {
+	tsk_error_print(stderr);
+	fs->close(fs);
+	img->close(img);
+	exit(1);
+
+    }
 
     fs->close(fs);
     img->close(img);
