@@ -2,12 +2,13 @@
 ** ext2fs_dent
 ** The Sleuth Kit 
 **
-** $Date: 2005/10/13 04:15:21 $
+** $Date: 2006/04/13 16:05:25 $
 **
-** Human Interface Layer Support for an EXT2FS image
+** File name layer support for an EXT2FS image
 **
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
+** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
+** Copyright (c) 2003-2006 Brian Carrier.  All rights reserved 
 **
 ** TASK
 ** Copyright (c) 2002 Brian Carrier, @stake Inc.  All rights reserved
@@ -42,12 +43,13 @@ typedef struct {
 } EXT2FS_DINFO;
 
 
-static void ext2fs_dent_walk_lcl(FS_INFO *, EXT2FS_DINFO *, INUM_T, int,
-				 FS_DENT_WALK_FN, void *);
+static uint8_t ext2fs_dent_walk_lcl(FS_INFO *, EXT2FS_DINFO *, INUM_T, int,
+    FS_DENT_WALK_FN, void *);
 
-static void
+/* return 1 on error and 0 on success */
+static uint8_t
 ext2fs_dent_copy(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
-		 char *ext2_dent, FS_DENT * fs_dent)
+    char *ext2_dent, FS_DENT * fs_dent)
 {
     FS_INFO *fs = &(ext2fs->fs_info);
 
@@ -57,9 +59,14 @@ ext2fs_dent_copy(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
 	fs_dent->inode = getu32(fs, dir->inode);
 
 	/* ext2 does not null terminate */
-	if (getu16(fs, dir->name_len) >= fs_dent->name_max)
-	    error("ext2fs_dent_copy: Name Space too Small %d %d",
-		  getu16(fs, dir->name_len), fs_dent->name_max);
+	if (getu16(fs, dir->name_len) >= fs_dent->name_max) {
+	    tsk_errno = TSK_ERR_FS_ARG;
+	    snprintf(tsk_errstr, TSK_ERRSTR_L,
+		"ext2fs_dent_copy: Name Space too Small %d %lu",
+		getu16(fs, dir->name_len), fs_dent->name_max);
+	    tsk_errstr2[0] = '\0';
+	    return 1;
+	}
 
 	/* Copy and Null Terminate */
 	strncpy(fs_dent->name, dir->name, getu16(fs, dir->name_len));
@@ -73,9 +80,14 @@ ext2fs_dent_copy(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
 	fs_dent->inode = getu32(fs, dir->inode);
 
 	/* ext2 does not null terminate */
-	if (dir->name_len >= fs_dent->name_max)
-	    error("ext2_dent_copy: Name Space too Small %d %d",
-		  dir->name_len, fs_dent->name_max);
+	if (dir->name_len >= fs_dent->name_max) {
+	    tsk_errno = TSK_ERR_FS_ARG;
+	    snprintf(tsk_errstr, TSK_ERRSTR_L,
+		"ext2_dent_copy: Name Space too Small %d %lu",
+		dir->name_len, fs_dent->name_max);
+	    tsk_errstr2[0] = '\0';
+	    return 1;
+	}
 
 	/* Copy and Null Terminate */
 	strncpy(fs_dent->name, dir->name, dir->name_len);
@@ -113,16 +125,24 @@ ext2fs_dent_copy(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
     fs_dent->path = dinfo->dirs;
     fs_dent->pathdepth = dinfo->depth;
 
-    if ((fs != NULL) && (fs_dent->inode)) {
+    if ((fs != NULL) && (fs_dent->inode)
+	&& (fs_dent->inode <= fs->last_inum)) {
 	/* Get inode */
 	if (fs_dent->fsi)
 	    fs_inode_free(fs_dent->fsi);
 
-	fs_dent->fsi = fs->inode_lookup(fs, fs_dent->inode);
+	if ((fs_dent->fsi = fs->inode_lookup(fs, fs_dent->inode)) == NULL) {
+	    strncat(tsk_errstr2, " - ext2fs_dent_copy",
+		TSK_ERRSTR_L - strlen(tsk_errstr2));
+	    return 1;
+	}
     }
     else {
+	if (fs_dent->fsi)
+	    fs_inode_free(fs_dent->fsi);
 	fs_dent->fsi = NULL;
     }
+    return 0;
 }
 
 
@@ -134,12 +154,11 @@ ext2fs_dent_copy(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
 ** it is deleted then call action with FS_FLAG_NAME_UNALLOC.
 ** len is the size of buf
 **
-** return how much was read this time or 0 if action wanted to stop
+** return 1 to stop, 0 on success, and -1 on error
 */
 static int
 ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
-			char *buf, int len, int flags,
-			FS_DENT_WALK_FN action, void *ptr)
+    char *buf, int len, int flags, FS_DENT_WALK_FN action, void *ptr)
 {
     FS_INFO *fs = &(ext2fs->fs_info);
 
@@ -151,7 +170,8 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
     FS_DENT *fs_dent;
     int minreclen = 4;
 
-    fs_dent = fs_dent_alloc(EXT2FS_MAXNAMLEN + 1, 0);
+    if ((fs_dent = fs_dent_alloc(EXT2FS_MAXNAMLEN + 1, 0)) == NULL)
+	return -1;
 
     /* update each time by the actual length instead of the
      ** recorded length so we can view the deleted entries 
@@ -178,7 +198,7 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
 	minreclen = EXT2FS_DIRSIZ_lcl(namelen);
 
 	/* 
-	 ** Check if we may have a valid direct entry.  If we don't
+	 ** Check if we may have a valid directory entry.  If we don't,
 	 ** then increment to the next word and try again.  
 	 */
 	if ((inode > fs->last_inum) ||
@@ -187,7 +207,6 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
 	    (namelen <= 0) ||
 	    (reclen < minreclen) || (reclen % 4) || (idx + reclen > len)) {
 
-	    /* we don't have a valid entry, skip ahead 4 */
 	    minreclen = 4;
 	    if (dellen > 0)
 		dellen -= 4;
@@ -198,21 +217,33 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
 	 * sure that it also ends in the unalloc space */
 	if ((dellen) && (dellen < minreclen)) {
 	    minreclen = 4;
-	    dellen -= 4;
+	    if (dellen > 0)
+		dellen -= 4;
 	    continue;
 	}
 
-	ext2fs_dent_copy(ext2fs, dinfo, dirPtr, fs_dent);
+	if (ext2fs_dent_copy(ext2fs, dinfo, dirPtr, fs_dent)) {
+	    fs_dent_free(fs_dent);
+	    return -1;
+	}
 
 	myflags = 0;
 	/* Do we have a deleted entry? */
 	if ((dellen > 0) || (inode == 0)) {
 	    myflags |= FS_FLAG_NAME_UNALLOC;
-	    dellen -= minreclen;
+	    if (dellen > 0)
+		dellen -= minreclen;
+
 	    if (flags & FS_FLAG_NAME_UNALLOC) {
-		if (WALK_STOP == action(fs, fs_dent, myflags, ptr)) {
+		int retval;
+		retval = action(fs, fs_dent, myflags, ptr);
+		if (retval == WALK_STOP) {
 		    fs_dent_free(fs_dent);
-		    return 0;
+		    return 1;
+		}
+		else if (retval == WALK_ERROR) {
+		    fs_dent_free(fs_dent);
+		    return -1;
 		}
 	    }
 	}
@@ -220,9 +251,16 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
 	else {
 	    myflags |= FS_FLAG_NAME_ALLOC;
 	    if (flags & FS_FLAG_NAME_ALLOC) {
-		if (WALK_STOP == action(fs, fs_dent, myflags, ptr)) {
+		int retval;
+
+		retval = action(fs, fs_dent, myflags, ptr);
+		if (retval == WALK_STOP) {
 		    fs_dent_free(fs_dent);
-		    return 0;
+		    return 1;
+		}
+		else if (retval == WALK_ERROR) {
+		    fs_dent_free(fs_dent);
+		    return -1;
 		}
 	    }
 	}
@@ -252,12 +290,26 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
 		dinfo->didx[dinfo->depth] =
 		    &dinfo->dirs[strlen(dinfo->dirs)];
 		strncpy(dinfo->didx[dinfo->depth], fs_dent->name,
-			DIR_STRSZ - strlen(dinfo->dirs));
+		    DIR_STRSZ - strlen(dinfo->dirs));
 		strncat(dinfo->dirs, "/", DIR_STRSZ);
 	    }
 	    dinfo->depth++;
-	    ext2fs_dent_walk_lcl(&(ext2fs->fs_info), dinfo,
-				 fs_dent->inode, flags, action, ptr);
+	    if (ext2fs_dent_walk_lcl(&(ext2fs->fs_info), dinfo,
+		    fs_dent->inode, flags, action, ptr)) {
+		/* If this fails because the directory could not be 
+		 * loaded, then we still continue */
+		if (verbose) {
+		    fprintf(stderr,
+			"ffs_dent_parse_block: error reading directory: %"
+			PRIuINUM "\n", fs_dent->inode);
+		    tsk_error_print(stderr);
+		}
+
+		tsk_errno = 0;
+		tsk_errstr[0] = '\0';
+		tsk_errstr2[0] = '\0';
+	    }
+
 
 	    dinfo->depth--;
 	    if (dinfo->depth < MAX_DEPTH)
@@ -266,7 +318,7 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
     }
 
     fs_dent_free(fs_dent);
-    return len;
+    return 0;
 }				/* end ext2fs_dent_parse_block() */
 
 
@@ -276,83 +328,108 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
 ** action is called for each entry with flags set to FS_FLAG_NAME_ALLOC for
 ** active entries
 **
-** this calls ext2fs_dent_walk_direct and _indirect
+** this calls ext2fs_dent_parse_block to do the actual analysis
 **
 ** Use the following flags: FS_FLAG_NAME_ALLOC, FS_FLAG_NAME_UNALLOC, 
 ** FS_FLAG_NAME_RECURSE
+**
+** returns 0 on success and 1 on error
 */
-void
+uint8_t
 ext2fs_dent_walk(FS_INFO * fs, INUM_T inode, int flags,
-		 FS_DENT_WALK_FN action, void *ptr)
+    FS_DENT_WALK_FN action, void *ptr)
 {
     EXT2FS_DINFO dinfo;
 
     memset(&dinfo, 0, sizeof(EXT2FS_DINFO));
-    ext2fs_dent_walk_lcl(fs, &dinfo, inode, flags, action, ptr);
+    return ext2fs_dent_walk_lcl(fs, &dinfo, inode, flags, action, ptr);
 }
 
-static void
+/* returns 0 on success and 1 on error */
+static uint8_t
 ext2fs_dent_walk_lcl(FS_INFO * fs, EXT2FS_DINFO * dinfo, INUM_T inode,
-		     int flags, FS_DENT_WALK_FN action, void *ptr)
+    int flags, FS_DENT_WALK_FN action, void *ptr)
 {
     FS_INODE *fs_inode;
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
     char *dirbuf, *dirptr;
     OFF_T size;
     FS_LOAD_FILE load_file;
+    int retval = 0;
 
-    if (inode < fs->first_inum || inode > fs->last_inum)
-	error("invalid inode value: %" PRIuINUM "\n", inode);
+    if (inode < fs->first_inum || inode > fs->last_inum) {
+	tsk_errno = TSK_ERR_FS_WALK_RNG;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "ext2fs_dent_walk_lcl: inode value: %" PRIuINUM "\n", inode);
+	tsk_errstr2[0] = '\0';
+	return 1;
+    }
 
     if (verbose)
 	fprintf(stderr,
-		"ext2fs_dent_walk: Processing directory %" PRIuINUM "\n",
-		inode);
+	    "ext2fs_dent_walk_lcl: Processing directory %" PRIuINUM
+	    "\n", inode);
+
+    if ((fs_inode = fs->inode_lookup(fs, inode)) == NULL) {
+	strncat(tsk_errstr2, " - ext2fs_dent_walk_lcl",
+	    TSK_ERRSTR_L - strlen(tsk_errstr2));
+	return 1;
+    }
 
-    fs_inode = fs->inode_lookup(fs, inode);
     size = roundup(fs_inode->size, fs->block_size);
-    dirbuf = mymalloc(size);
+    if ((dirbuf = mymalloc(size)) == NULL) {
+	fs_inode_free(fs_inode);
+	return 1;
+    }
 
     /* make a copy of the directory contents that we can process */
     load_file.left = load_file.total = size;
     load_file.base = load_file.cur = dirbuf;
 
-    fs->file_walk(fs, fs_inode, 0, 0,
-		  FS_FLAG_FILE_SLACK | FS_FLAG_FILE_NOID |
-		  FS_FLAG_FILE_NOABORT, load_file_action,
-		  (void *) &load_file);
+    if (fs->file_walk(fs, fs_inode, 0, 0,
+	    FS_FLAG_FILE_SLACK | FS_FLAG_FILE_NOID,
+	    load_file_action, (void *) &load_file)) {
+	free(dirbuf);
+	fs_inode_free(fs_inode);
+	strncat(tsk_errstr2, " - extX_dent_walk_lcl",
+	    TSK_ERRSTR_L - strlen(tsk_errstr2));
+
+	return 1;
+    }
 
     /* Not all of the directory was copied, so we exit */
     if (load_file.left > 0) {
 	free(dirbuf);
 	fs_inode_free(fs_inode);
-
-	if (dinfo->depth == 0)
-	    error("Error reading directory contents: %" PRIuINUM "\n",
-		  inode);
-
-	return;
+	tsk_errno = TSK_ERR_FS_FWALK;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "ext2fs_dent_walk: Error reading directory contents: %"
+	    PRIuINUM "\n", inode);
+	tsk_errstr2[0] = '\0';
+	return 1;
     }
-
     dirptr = dirbuf;
 
     while (size > 0) {
 	int len = (fs->block_size < size) ? fs->block_size : size;
 
-	int retval =
+	retval =
 	    ext2fs_dent_parse_block(ext2fs, dinfo, dirptr, len, flags,
-				    action, ptr);
+	    action, ptr);
 
-	/* if 0, then the action wants to stop */
-	if (retval)
-	    size -= retval;
-	else
+	/* if 1, then the action wants to stop, -1 is error */
+	if ((retval == 1) || (retval == -1))
 	    break;
 
+	size -= len;
 	dirptr = (char *) ((uintptr_t) dirptr + len);
     }
 
     fs_inode_free(fs_inode);
     free(dirbuf);
-    return;
+
+    if (retval == -1)
+	return 1;
+    else
+	return 0;
 }
