@@ -2,11 +2,12 @@
 ** ffind  (file find)
 ** The Sleuth Kit 
 **
-** $Date: 2005/09/02 23:34:02 $
+** $Date: 2006/04/06 01:15:55 $
 **
 ** Find the file that uses the specified inode (including deleted files)
 ** 
 ** Brian Carrier [carrier@sleuthkit.org]
+** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
 **
 ** TASK
@@ -25,18 +26,17 @@ void
 usage()
 {
     fprintf(stderr,
-	    "usage: %s [-aduvV] [-f fstype] [-i imgtype] [-o imgoffset] image [images] inode\n",
-	    progname);
+	"usage: %s [-aduvV] [-f fstype] [-i imgtype] [-o imgoffset] image [images] inode\n",
+	progname);
     fprintf(stderr, "\t-a: Find all occurrences\n");
     fprintf(stderr, "\t-d: Find deleted entries ONLY\n");
     fprintf(stderr, "\t-u: Find undeleted entries ONLY\n");
     fprintf(stderr, "\t-i imgtype: The format of the image file\n");
     fprintf(stderr,
-	    "\t-o imgoffset: The offset of the file system in the image (in sectors)\n");
+	"\t-o imgoffset: The offset of the file system in the image (in sectors)\n");
     fprintf(stderr, "\t-v: Verbose output to stderr\n");
     fprintf(stderr, "\t-V: Print version\n");
     fprintf(stderr, "\t-f fstype: Image file system type\n");
-    fprintf(stderr, "Supported file system types:\n");
     fs_print_types(stderr);
     fprintf(stderr, "Supported image format types:\n");
     img_print_types(stderr);
@@ -56,10 +56,11 @@ main(int argc, char **argv)
     extern int optind;
     uint32_t type;
     uint16_t id;
-    char *imgtype = NULL, *imgoff = NULL, *dash;
+    char *imgtype = NULL, *dash;
     IMG_INFO *img;
     uint8_t localflags = 0;
     INUM_T inode;
+    SSIZE_T imgoff = 0;
 
     progname = argv[0];
     setlocale(LC_ALL, "");
@@ -79,7 +80,10 @@ main(int argc, char **argv)
 	    imgtype = optarg;
 	    break;
 	case 'o':
-	    imgoff = optarg;
+	    if ((imgoff = parse_offset(optarg)) == -1) {
+		tsk_error_print(stderr);
+		exit(1);
+	    }
 	    break;
 	case 'u':
 	    flags |= FS_FLAG_NAME_ALLOC;
@@ -111,8 +115,6 @@ main(int argc, char **argv)
     }
 
 
-
-
     /* we have the inum-type or inum-type-id format */
     type = 0;
     id = 0;
@@ -147,23 +149,38 @@ main(int argc, char **argv)
 
 
     /* open image */
-    img =
-	img_open(imgtype, imgoff, argc - optind - 1,
-		 (const char **) &argv[optind]);
-    fs = fs_open(img, fstype);
+    if ((img =
+	    img_open(imgtype, argc - optind - 1,
+		(const char **) &argv[optind])) == NULL) {
+	tsk_error_print(stderr);
+	exit(1);
+    }
+    if ((fs = fs_open(img, imgoff, fstype)) == NULL) {
+	tsk_error_print(stderr);
+	if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
+	    fs_print_types(stderr);
+	img->close(img);
+	exit(1);
+    }
+
 
     if (inode < fs->first_inum) {
 	fprintf(stderr, "Inode is too small for image (%" PRIuINUM ")\n",
-		fs->first_inum);
+	    fs->first_inum);
 	exit(1);
     }
     if (inode > fs->last_inum) {
 	fprintf(stderr, "Inode is too large for image (%" PRIuINUM ")\n",
-		fs->last_inum);
+	    fs->last_inum);
 	exit(1);
     }
 
-    fs_ffind(fs, localflags, inode, type, id, flags);
+    if (fs_ffind(fs, localflags, inode, type, id, flags)) {
+	tsk_error_print(stderr);
+	fs->close(fs);
+	img->close(img);
+	exit(1);
+    }
 
     fs->close(fs);
     img->close(img);
