@@ -2,7 +2,7 @@
 ** fscheck
 ** The Sleuth Kit 
 **
-** $Date: 2005/09/02 23:34:03 $
+** $Date: 2006/03/27 03:33:19 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2004-2005 Brian Carrier.  All rights reserved 
@@ -18,15 +18,14 @@ static void
 usage()
 {
     fprintf(stderr,
-	    "usage: %s [-vV] [-f fstype] [-i imgtype] [-o imgoffset] image [images]\n",
-	    progname);
+	"usage: %s [-vV] [-f fstype] [-i imgtype] [-o imgoffset] image [images]\n",
+	progname);
     fprintf(stderr, "\t-i imgtype: The format of the image file\n");
     fprintf(stderr,
-	    "\t-o imgoffset: The offset of the file system in the image (in sectors)\n");
+	"\t-o imgoffset: The offset of the file system in the image (in sectors)\n");
     fprintf(stderr, "\t-v: verbose output to stderr\n");
     fprintf(stderr, "\t-V: Print version\n");
     fprintf(stderr, "\t-f fstype: File system type\n");
-    fprintf(stderr, "Supported file system types:\n");
     fs_print_types(stderr);
     fprintf(stderr, "Supported image format types:\n");
     img_print_types(stderr);
@@ -39,7 +38,7 @@ int
 main(int argc, char **argv)
 {
     char *fstype = NULL;
-    char ch;
+    int ch;
     FS_INFO *fs;
     char *imgtype = NULL, *imgoff = NULL;
     IMG_INFO *img;
@@ -83,10 +82,28 @@ main(int argc, char **argv)
 
     img =
 	img_open(imgtype, imgoff, argc - optind,
-		 (const char **) &argv[optind]);
-    fs = fs_open(img, fstype);
+	(const char **) &argv[optind]);
+    if (img == NULL) {
+	tsk_error_print(stderr);
+	exit(1);
+    }
 
-    fs->fscheck(fs, stdout);
+    if (fs = fs_open(img, fstype)) {
+	if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
+	    tsk_print_types(stderr);
+
+	tsk_error_print(stderr);
+	img->close(img);
+	exit(1);
+
+    }
+
+    if (fs->fscheck(fs, stdout)) {
+	tsk_error_print(stderr);
+	fs->close(fs);
+	img->close(img);
+	exit(1);
+    }
 
     fs->close(fs);
     img->close(img);
