@@ -2,11 +2,12 @@
 ** ifind (inode find)
 ** The Sleuth Kit
 **
-** $Date: 2005/09/02 23:34:03 $
+** $Date: 2006/04/07 18:12:00 $
 **
 ** Given an image  and block number, identify which inode it is used by
 ** 
 ** Brian Carrier [carrier@sleuthkit.org]
+** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
 **
 ** TASK
@@ -23,7 +24,6 @@
 #include "libfstools.h"
 
 
-
 static uint8_t localflags;
 static uint8_t found;
 
@@ -52,11 +52,11 @@ ifind_par_act(FS_INFO * fs, FS_INODE * fs_inode, int flags, void *ptr)
 	    strncpy(fs_dent->name, fs_name->name, fs_dent->name_max);
 	    if (localflags & IFIND_PAR_LONG) {
 		fs_dent_print_long(stdout, fs_dent, FS_FLAG_NAME_UNALLOC,
-				   fs, NULL);
+		    fs, NULL);
 	    }
 	    else {
 		fs_dent_print(stdout, fs_dent, FS_FLAG_NAME_UNALLOC, fs,
-			      NULL);
+		    NULL);
 		printf("\n");
 	    }
 	    fs_dent->fsi = NULL;
@@ -70,7 +70,7 @@ ifind_par_act(FS_INFO * fs, FS_INODE * fs_inode, int flags, void *ptr)
 
 
 
-
+/* return 1 on error and 0 on success */
 uint8_t
 fs_ifind_par(FS_INFO * fs, uint8_t lclflags, INUM_T par)
 {
@@ -78,22 +78,23 @@ fs_ifind_par(FS_INFO * fs, uint8_t lclflags, INUM_T par)
     localflags = lclflags;
     parinode = par;
     fs_dent = fs_dent_alloc(256, 0);
+    if (fs_dent == NULL)
+	return 1;
 
-    fs->inode_walk(fs, fs->first_inum, fs->last_inum,
-		   FS_FLAG_META_LINK | FS_FLAG_META_UNLINK |
-		   FS_FLAG_META_UNALLOC | FS_FLAG_META_USED,
-		   ifind_par_act, NULL);
+    if (fs->inode_walk(fs, fs->first_inum, fs->last_inum,
+	    FS_FLAG_META_LINK | FS_FLAG_META_UNLINK |
+	    FS_FLAG_META_UNALLOC | FS_FLAG_META_USED, ifind_par_act,
+	    NULL)) {
+	fs_dent_free(fs_dent);
+	return 1;
+    }
 
     fs_dent_free(fs_dent);
-
     return 0;
 }
 
 
 
-
-
-
 /*******************************************************************************
  * Find an inode given a file path
  */
@@ -115,9 +116,10 @@ ifind_path_act(FS_INFO * fs, FS_DENT * fs_dent, int flags, void *ptr)
      * it got that way, so this was added
      */
     if (cur_dir == NULL) {
-	fprintf(stderr,
-		"cur_dir is null: Please run with '-v' and send output to developers\n");
-	return WALK_STOP;
+	tsk_errno = TSK_ERR_FS_ARG;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "ifind: cur_dir is null: Please run with '-v' and send output to developers\n");
+	return WALK_ERROR;
     }
 
     /* 
@@ -158,7 +160,7 @@ ifind_path_act(FS_INFO * fs, FS_DENT * fs_dent, int flags, void *ptr)
 	    }
 	    if (fail) {
 		printf("Attribute name (%s) not found in %s: %" PRIuINUM
-		       "\n", cur_attr, cur_dir, fs_dent->inode);
+		    "\n", cur_attr, cur_dir, fs_dent->inode);
 
 		return WALK_STOP;
 	    }
@@ -181,7 +183,7 @@ ifind_path_act(FS_INFO * fs, FS_DENT * fs_dent, int flags, void *ptr)
 
     if (verbose)
 	fprintf(stderr, "Found it (%s), now looking for %s\n",
-		fs_dent->name, cur_dir);
+	    fs_dent->name, cur_dir);
 
     /* That was the last one */
     if (cur_dir == NULL) {
@@ -202,9 +204,11 @@ ifind_path_act(FS_INFO * fs, FS_DENT * fs_dent, int flags, void *ptr)
     /* it is a directory so we can recurse */
     if ((fs_dent->fsi->mode & FS_INODE_FMT) == FS_INODE_DIR) {
 
-	fs->dent_walk(fs, fs_dent->inode,
-		      FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_UNALLOC,
-		      ifind_path_act, (char *) 0);
+	if (fs->dent_walk(fs, fs_dent->inode,
+		FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_UNALLOC,
+		ifind_path_act, (char *) 0)) {
+	    return WALK_ERROR;
+	}
     }
 
     /* The name was correct, but it was not a directory */
@@ -216,6 +220,7 @@ ifind_path_act(FS_INFO * fs, FS_DENT * fs_dent, int flags, void *ptr)
 }
 
 
+/* Return 1 for error, 0 if no error */
 uint8_t
 fs_ifind_path(FS_INFO * fs, uint8_t lclflags, char *path)
 {
@@ -241,13 +246,14 @@ fs_ifind_path(FS_INFO * fs, uint8_t lclflags, char *path)
     if (verbose)
 	fprintf(stderr, "Looking for %s\n", cur_dir);
 
-    fs->dent_walk(fs, fs->root_inum,
-		  FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_UNALLOC,
-		  ifind_path_act, NULL);
+    if (fs->dent_walk(fs, fs->root_inum,
+	    FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_UNALLOC, ifind_path_act,
+	    NULL)) {
+	return 1;
+    }
 
     if (0 == found) {
 	printf("File not found: %s\n", cur_dir);
-	return 1;
     }
     return 0;
 }
@@ -271,7 +277,7 @@ static uint16_t curid;
  */
 static uint8_t
 ifind_data_file_act(FS_INFO * fs, DADDR_T addr, char *buf,
-		    unsigned int size, int flags, void *ptr)
+    unsigned int size, int flags, void *ptr)
 {
     /* Drop references to block zero (sparse)
      * This becomes an issue with fragments and looking for fragments
@@ -301,11 +307,11 @@ ifind_data_file_act(FS_INFO * fs, DADDR_T addr, char *buf,
  */
 static uint8_t
 ifind_data_file_ntfs_act(FS_INFO * fs, DADDR_T addr, char *buf,
-			 unsigned int size, int flags, void *ptr)
+    unsigned int size, int flags, void *ptr)
 {
     if (addr == block) {
 	printf("%" PRIuINUM "-%" PRIu32 "-%" PRIu16 "\n", curinode,
-	       curtype, curid);
+	    curtype, curid);
 
 	if (!(localflags & IFIND_ALL)) {
 	    fs->close(fs);
@@ -326,7 +332,7 @@ ifind_data_file_ntfs_act(FS_INFO * fs, DADDR_T addr, char *buf,
 static uint8_t
 ifind_data_act(FS_INFO * fs, FS_INODE * fs_inode, int flags, void *ptr)
 {
-    int file_flags = (FS_FLAG_FILE_AONLY | FS_FLAG_FILE_NOABORT);
+    int file_flags = (FS_FLAG_FILE_AONLY);
 
     /* If the meta data structure is unallocated, then set the recovery flag */
     if (flags & FS_FLAG_META_UNALLOC)
@@ -343,8 +349,14 @@ ifind_data_act(FS_INFO * fs, FS_INODE * fs_inode, int flags, void *ptr)
 	    curtype = data->type;
 	    curid = data->id;
 	    if (data->flags & FS_DATA_NONRES) {
-		fs->file_walk(fs, fs_inode, data->type, data->id,
-			      file_flags, ifind_data_file_ntfs_act, ptr);
+		if (fs->file_walk(fs, fs_inode, data->type, data->id,
+			file_flags, ifind_data_file_ntfs_act, ptr)) {
+		    if (verbose)
+			fprintf(stderr, "Error walking file %" PRIuINUM,
+			    fs_inode->addr);
+/* Ignore these errors */
+		    tsk_errno = 0;
+		}
 	    }
 	    data = data->next;
 	}
@@ -352,17 +364,31 @@ ifind_data_act(FS_INFO * fs, FS_INODE * fs_inode, int flags, void *ptr)
     }
     else if ((fs->ftype & FSMASK) == FATFS_TYPE) {
 	file_flags |= (FS_FLAG_FILE_SLACK | FS_FLAG_FILE_NOID);
-	fs->file_walk(fs, fs_inode, 0, 0, file_flags, ifind_data_file_act,
-		      ptr);
+	if (fs->file_walk(fs, fs_inode, 0, 0, file_flags,
+		ifind_data_file_act, ptr)) {
+	    if (verbose)
+		fprintf(stderr, "Error walking file %" PRIuINUM,
+		    fs_inode->addr);
+/* Ignore these errors */
+	    tsk_errno = 0;
+	}
     }
     /* UNIX do not need the SLACK flag because they use fragments - if the
      * SLACK flag exists then any unused fragments in a block will be 
      * correlated with the incorrect inode
+     *
+     * The META flag is needed though to find indirect blocks
      */
     else {
-	file_flags |= (FS_FLAG_FILE_NOID);
-	fs->file_walk(fs, fs_inode, 0, 0, file_flags, ifind_data_file_act,
-		      ptr);
+	file_flags |= (FS_FLAG_FILE_NOID | FS_FLAG_FILE_META);
+	if (fs->file_walk(fs, fs_inode, 0, 0, file_flags,
+		ifind_data_file_act, ptr)) {
+	    if (verbose)
+		fprintf(stderr, "Error walking file %" PRIuINUM,
+		    fs_inode->addr);
+/* Ignore these errors */
+	    tsk_errno = 0;
+	}
     }
 
     return WALK_CONT;
@@ -375,44 +401,47 @@ ifind_data_act(FS_INFO * fs, FS_INODE * fs_inode, int flags, void *ptr)
  */
 static uint8_t
 ifind_data_block_act(FS_INFO * fs, DADDR_T addr, char *buf, int flags,
-		     void *ptr)
+    void *ptr)
 {
-    if (flags & FS_FLAG_DATA_META)
+    if (flags & FS_FLAG_DATA_META) {
 	printf("Meta Data\n");
-    else
-	printf("Inode not found\n");
+	found = 1;
+    }
 
     return WALK_STOP;
 }
 
 
+/* Return 1 on error, 0 if no error */
 uint8_t
 fs_ifind_data(FS_INFO * fs, uint8_t lclflags, DADDR_T blk)
 {
-
     found = 0;
     localflags = lclflags;
     block = blk;
 
-    fs->inode_walk(fs, fs->first_inum, fs->last_inum,
-		   FS_FLAG_META_LINK | FS_FLAG_META_UNLINK |
-		   FS_FLAG_META_ALLOC | FS_FLAG_META_UNALLOC |
-		   FS_FLAG_META_USED | FS_FLAG_META_UNUSED,
-		   ifind_data_act, NULL);
+    if (fs->inode_walk(fs, fs->first_inum, fs->last_inum,
+	    FS_FLAG_META_LINK | FS_FLAG_META_UNLINK |
+	    FS_FLAG_META_ALLOC | FS_FLAG_META_UNALLOC |
+	    FS_FLAG_META_USED | FS_FLAG_META_UNUSED, ifind_data_act,
+	    NULL)) {
+	return 1;
+    }
 
     /* 
      * If we did not find an inode yet, we call block_walk for the 
      * block to find out the associated flags so we can identify it as
      * a meta data block */
     if (0 == found) {
-	fs->block_walk(fs, block, block,
-		       FS_FLAG_DATA_UNALLOC | FS_FLAG_DATA_ALLOC |
-		       FS_FLAG_DATA_META | FS_FLAG_DATA_CONT,
-		       ifind_data_block_act, NULL);
+	if (fs->block_walk(fs, block, block,
+		FS_FLAG_DATA_UNALLOC | FS_FLAG_DATA_ALLOC |
+		FS_FLAG_DATA_META | FS_FLAG_DATA_CONT,
+		ifind_data_block_act, NULL)) {
+	    return 1;
+	}
     }
     if (0 == found) {
 	printf("Inode not found\n");
-	return 1;
     }
     return 0;
 }
