@@ -2,11 +2,12 @@
 ** istat
 ** The Sleuth Kit 
 **
-** $Date: 2005/09/02 23:34:03 $
+** $Date: 2006/05/08 17:56:51 $
 **
 ** Display all inode info about a given inode.
 **
 ** Brian Carrier [carrier@sleuthkit.org]
+** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
 **
 ** TASK
@@ -22,30 +23,6 @@
 
 #include "libfstools.h"
 
-/* atoinum - convert string to inode number */
-INUM_T
-atoinum(const char *str)
-{
-    char *cp, *dash;
-    INUM_T inum;
-
-    if (*str == 0)
-	return (0);
-
-    /* if we are given the inode in the inode-type-id form, then ignore
-     * the other stuff w/out giving an error 
-     *
-     * This will make scripting easier
-     */
-    if ((dash = strchr(str, '-')) != NULL) {
-	*dash = '\0';
-    }
-
-    inum = strtoull(str, &cp, 0);
-    if (*cp || cp == str)
-	error("bad inode number: %s", str);
-    return (inum);
-}
 
 /* usage - explain and terminate */
 
@@ -53,21 +30,20 @@ static void
 usage()
 {
     fprintf(stderr,
-	    "usage: %s [-b num] [-f fstype] [-i imgtype] [-o imgoffset] [-z zone] [-s seconds] [-vV] image inum\n",
-	    progname);
+	"usage: %s [-b num] [-f fstype] [-i imgtype] [-o imgoffset] [-z zone] [-s seconds] [-vV] image inum\n",
+	progname);
     fprintf(stderr,
-	    "\t-b num: force the display of NUM address of block pointers\n");
+	"\t-b num: force the display of NUM address of block pointers\n");
     fprintf(stderr,
-	    "\t-z zone: time zone of original machine (i.e. EST5EDT or GMT)\n");
+	"\t-z zone: time zone of original machine (i.e. EST5EDT or GMT)\n");
     fprintf(stderr,
-	    "\t-s seconds: Time skew of original machine (in seconds)\n");
+	"\t-s seconds: Time skew of original machine (in seconds)\n");
     fprintf(stderr, "\t-i imgtype: The format of the image file\n");
     fprintf(stderr,
-	    "\t-o imgoffset: The offset of the file system in the image (in sectors)\n");
+	"\t-o imgoffset: The offset of the file system in the image (in sectors)\n");
     fprintf(stderr, "\t-v: verbose output to stderr\n");
     fprintf(stderr, "\t-V: print version\n");
     fprintf(stderr, "\t-f fstype: File system type\n");
-    fprintf(stderr, "Supported file system types:\n");
     fs_print_types(stderr);
     fprintf(stderr, "Supported image format types:\n");
     img_print_types(stderr);
@@ -80,12 +56,13 @@ main(int argc, char **argv)
 {
     INUM_T inum;
     int ch;
-    char *cp;
+    char *cp, *dash;
     char *fstype = NULL;
     FS_INFO *fs;
     int32_t sec_skew = 0;
-    char *imgtype = NULL, *imgoff = NULL;
+    char *imgtype = NULL;
     IMG_INFO *img;
+    SSIZE_T imgoff = 0;
 
 
     /* When > 0 this is the number of blocks to print, used for -b arg */
@@ -105,8 +82,8 @@ main(int argc, char **argv)
 	    numblock = strtoull(optarg, &cp, 0);
 	    if (*cp || cp == optarg || numblock < 1) {
 		fprintf(stderr,
-			"invalid argument: block count must be positive: %s\n",
-			optarg);
+		    "invalid argument: block count must be positive: %s\n",
+		    optarg);
 		usage();
 	    }
 	    break;
@@ -118,7 +95,10 @@ main(int argc, char **argv)
 	    break;
 
 	case 'o':
-	    imgoff = optarg;
+	    if ((imgoff = parse_offset(optarg)) == -1) {
+		tsk_error_print(stderr);
+		exit(1);
+	    }
 	    break;
 
 	case 's':
@@ -137,7 +117,8 @@ main(int argc, char **argv)
 		char envstr[32];
 		snprintf(envstr, 32, "TZ=%s", optarg);
 		if (0 != putenv(envstr)) {
-		    error("error setting environment");
+		    fprintf(stderr, "error setting environment");
+		    exit(1);
 		}
 
 		tzset();
@@ -153,31 +134,63 @@ main(int argc, char **argv)
 	usage();
     }
 
-    inum = atoinum(argv[argc - 1]);
+    /* if we are given the inode in the inode-type-id form, then ignore
+     * the other stuff w/out giving an error 
+     *
+     * This will make scripting easier
+     */
+    if ((dash = strchr(argv[argc - 1], '-')) != NULL) {
+	*dash = '\0';
+    }
+
+    inum = strtoull(argv[argc - 1], &cp, 0);
+    if (*cp || cp == argv[argc - 1]) {
+	fprintf(stderr, "bad inode number: %s", argv[argc - 1]);
+	usage();
+    }
 
     /*
      * Open the file system.
      */
-    img =
-	img_open(imgtype, imgoff, argc - optind - 1,
-		 (const char **) &argv[optind]);
-    fs = fs_open(img, fstype);
+    if ((img =
+	    img_open(imgtype, argc - optind - 1,
+		(const char **) &argv[optind])) == NULL) {
+	tsk_error_print(stderr);
+	exit(1);
+    }
+
+    if ((fs = fs_open(img, imgoff, fstype)) == NULL) {
+	tsk_error_print(stderr);
+	if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
+	    fs_print_types(stderr);
+	img->close(img);
+	exit(1);
+    }
 
     if (inum > fs->last_inum) {
 	fprintf(stderr,
-		"Inode value is too large for image (%" PRIuINUM ")\n",
-		fs->last_inum);
+	    "Metadata address is too large for image (%" PRIuINUM ")\n",
+	    fs->last_inum);
+	fs->close(fs);
+	img->close(img);
 	exit(1);
     }
 
     if (inum < fs->first_inum) {
 	fprintf(stderr,
-		"Inode value is too small for image (%" PRIuINUM ")\n",
-		fs->first_inum);
+	    "Metadata address is too small for image (%" PRIuINUM ")\n",
+	    fs->first_inum);
+	fs->close(fs);
+	img->close(img);
 	exit(1);
     }
 
-    fs->istat(fs, stdout, inum, numblock, sec_skew);
+    if (fs->istat(fs, stdout, inum, numblock, sec_skew)) {
+	tsk_error_print(stderr);
+	fs->close(fs);
+	img->close(img);
+	exit(1);
+    }
 
     fs->close(fs);
     img->close(img);
