@@ -2,9 +2,10 @@
 ** jcat
 ** The Sleuth Kit 
 **
-** $Date: 2005/09/02 23:34:03 $
+** $Date: 2006/04/06 01:15:56 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
+** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
 ** Copyright (c) 2004-2005 Brian Carrier.  All rights reserved
 **
 **
@@ -35,8 +36,10 @@ atoinum(const char *str)
 	*dash = '\0';
     }
     inum = strtoull(str, &cp, 0);
-    if (*cp || cp == str)
-	error("bad inode number: %s", str);
+    if (*cp || cp == str) {
+	fprintf(stderr, "bad inode number: %s", str);
+	exit(1);
+    }
     return (inum);
 }
 
@@ -46,18 +49,17 @@ static void
 usage()
 {
     fprintf(stderr,
-	    "usage: %s [-f fstype] [-i imgtype] [-o imgoffset] [-vV] image [images] [inode] blk\n",
-	    progname);
+	"usage: %s [-f fstype] [-i imgtype] [-o imgoffset] [-vV] image [images] [inode] blk\n",
+	progname);
     fprintf(stderr, "\tblk: The journal block to view\n");
     fprintf(stderr,
-	    "\tinode: The file system inode where the journal is located\n");
+	"\tinode: The file system inode where the journal is located\n");
     fprintf(stderr, "\t-i imgtype: The format of the image file\n");
     fprintf(stderr,
-	    "\t-o imgoffset: The offset of the file system in the image (in sectors)\n");
+	"\t-o imgoffset: The offset of the file system in the image (in sectors)\n");
     fprintf(stderr, "\t-v: verbose output to stderr\n");
     fprintf(stderr, "\t-V: print version\n");
     fprintf(stderr, "\t-f fstype: File system type\n");
-    fprintf(stderr, "Supported file system types:\n");
     fs_print_types(stderr);
     fprintf(stderr, "Supported image format types:\n");
     img_print_types(stderr);
@@ -74,8 +76,9 @@ main(int argc, char **argv)
     FS_INFO *fs;
     DADDR_T blk;
     char *cp;
-    char *imgtype = NULL, *imgoff = NULL;
+    char *imgtype = NULL;
     IMG_INFO *img;
+    SSIZE_T imgoff = 0;
 
     progname = argv[0];
     setlocale(LC_ALL, "");
@@ -94,7 +97,10 @@ main(int argc, char **argv)
 	    break;
 
 	case 'o':
-	    imgoff = optarg;
+	    if ((imgoff = parse_offset(optarg)) == -1) {
+		tsk_error_print(stderr);
+		exit(1);
+	    }
 	    break;
 	case 'v':
 	    verbose++;
@@ -108,54 +114,94 @@ main(int argc, char **argv)
 
     /* We need at least two more arguments */
     if (optind + 1 >= argc) {
-	fprintf(stderr, "Missing image name and/or address\n");
+	fprintf(stderr, "Missing image name and/or block address\n");
 	usage();
     }
 
     blk = strtoull(argv[argc - 1], &cp, 0);
-    if (*cp || cp == argv[argc - 1])
-	error("bad block number: %s", argv[argc - 1]);
+    if (*cp || cp == argv[argc - 1]) {
+	fprintf(stderr, "bad block number: %s", argv[argc - 1]);
+	exit(1);
+    }
 
     /* Do we have an inode as well? */
     inum = strtoull(argv[argc - 2], &cp, 0);
     if (*cp || cp == argv[argc - 2]) {
 	/* Not a number therefore an image */
 
-	img =
-	    img_open(imgtype, imgoff, argc - optind - 1,
-		     (const char **) &argv[optind]);
-	fs = fs_open(img, fstype);
+	if ((img =
+		img_open(imgtype, argc - optind - 1,
+		    (const char **) &argv[optind])) == NULL) {
+	    tsk_error_print(stderr);
+	    exit(1);
+	}
+
+	if ((fs = fs_open(img, imgoff, fstype)) == NULL) {
+	    tsk_error_print(stderr);
+	    if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
+		fs_print_types(stderr);
+	    img->close(img);
+	    exit(1);
+	}
+
 	inum = fs->journ_inum;
     }
     else {
-	img =
-	    img_open(imgtype, imgoff, argc - optind - 2,
-		     (const char **) &argv[optind]);
-	fs = fs_open(img, fstype);
+	if ((img =
+		img_open(imgtype, argc - optind - 2,
+		    (const char **) &argv[optind])) == NULL) {
+	    tsk_error_print(stderr);
+	    exit(1);
+	}
+
+	if ((fs = fs_open(img, imgoff, fstype)) == NULL) {
+	    tsk_error_print(stderr);
+	    if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
+		fs_print_types(stderr);
+	    img->close(img);
+	    exit(1);
+	}
+
     }
 
     if (inum > fs->last_inum) {
 	fprintf(stderr,
-		"Inode value is too large for image (%" PRIuINUM ")\n",
-		fs->last_inum);
+	    "Inode value is too large for image (%" PRIuINUM ")\n",
+	    fs->last_inum);
+	fs->close(fs);
+	img->close(img);
 	exit(1);
     }
 
     if (inum < fs->first_inum) {
 	fprintf(stderr,
-		"Inode value is too small for image (%" PRIuINUM ")\n",
-		fs->first_inum);
+	    "Inode value is too small for image (%" PRIuINUM ")\n",
+	    fs->first_inum);
+	fs->close(fs);
+	img->close(img);
 	exit(1);
     }
 
     if (fs->jopen == NULL) {
 	fprintf(stderr,
-		"Journal support does not exist for this file system\n");
+	    "Journal support does not exist for this file system\n");
+	fs->close(fs);
+	img->close(img);
 	return 1;
     }
 
-    fs->jopen(fs, inum);
-    fs->jblk_walk(fs, blk, blk, 0, 0, NULL);
+    if (fs->jopen(fs, inum)) {
+	tsk_error_print(stderr);
+	fs->close(fs);
+	img->close(img);
+	exit(1);
+    }
+    if (fs->jblk_walk(fs, blk, blk, 0, 0, NULL)) {
+	tsk_error_print(stderr);
+	fs->close(fs);
+	img->close(img);
+	exit(1);
+    }
 
     fs->close(fs);
     img->close(img);
