@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit
 **
-** $Date: 2005/09/02 19:53:28 $
+** $Date: 2006/03/27 03:33:20 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -490,17 +490,13 @@ extern "C" {
 
     } NTFS_INFO;
 
-    extern void
-     ntfs_data_walk(NTFS_INFO *, INUM_T, FS_DATA *, int, FS_FILE_WALK_FN,
-		    void *);
-
-    extern void
-     ntfs_dent_walk(FS_INFO *, INUM_T, int, FS_DENT_WALK_FN, void *);
-
+    extern uint8_t
+	ntfs_data_walk(NTFS_INFO *, INUM_T, FS_DATA *, int,
+	FS_FILE_WALK_FN, void *);
+    extern uint8_t
+	ntfs_dent_walk(FS_INFO *, INUM_T, int, FS_DENT_WALK_FN, void *);
     extern uint32_t nt2unixtime(uint64_t ntdate);
-
-    extern void
-     ntfs_attrname_lookup(FS_INFO *, uint16_t, char *, int);
+    extern uint8_t ntfs_attrname_lookup(FS_INFO *, uint16_t, char *, int);
 
 #ifdef __cplusplus
 }
