@@ -2,7 +2,7 @@
  * imgstat
  * The Sleuth Kit 
  *
- * $Date: 2005/09/02 23:34:04 $
+ * $Date: 2006/04/06 01:15:56 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2005 Brian Carrier.  All rights reserved 
@@ -32,7 +32,8 @@ int
 main(int argc, char **argv)
 {
     IMG_INFO *img;
-    char ch, *imgtype = NULL;
+    char *imgtype = NULL;
+    int ch;
     uint8_t type = 0;
 
     progname = argv[0];
@@ -68,8 +69,12 @@ main(int argc, char **argv)
 	usage();
     }
 
-    img =
-	img_open(imgtype, 0, argc - optind, (const char **) &argv[optind]);
+    if ((img =
+	    img_open(imgtype, argc - optind,
+		(const char **) &argv[optind])) == NULL) {
+	tsk_error_print(stderr);
+	exit(1);
+    }
 
     if (type) {
 	char *str = img_get_type(img->itype);
@@ -80,6 +85,5 @@ main(int argc, char **argv)
     }
 
     img->close(img);
-
     exit(0);
 }
