@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2005/09/02 19:53:28 $
+ * $Date: 2006/04/12 22:18:20 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2005 Brian Carrier.  All rights reserved 
@@ -13,7 +13,7 @@
 extern "C" {
 #endif
 
-    extern IMG_INFO *raw_open(OFF_T, const char **, IMG_INFO *);
+    extern IMG_INFO *raw_open(const char **, IMG_INFO *);
 
     typedef struct IMG_RAW_INFO IMG_RAW_INFO;
     struct IMG_RAW_INFO {
