@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2005/09/02 19:53:28 $
+ * $Date: 2006/04/12 22:18:20 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2005 Brian Carrier.  All rights reserved 
@@ -14,7 +14,7 @@
 extern "C" {
 #endif
 
-    extern IMG_INFO *split_open(OFF_T, int, const char **, IMG_INFO *);
+    extern IMG_INFO *split_open(int, const char **, IMG_INFO *);
 
 #define SPLIT_CACHE	15
 
