@@ -1,9 +1,10 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2005/09/02 23:34:04 $
+ * $Date: 2006/04/07 21:21:03 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
+ * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
  * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
  *
  * bsd.c: BSD Disk labels
@@ -24,6 +25,9 @@ static char *
 bsd_get_desc(uint8_t fstype)
 {
     char *str = mymalloc(64);
+    if (str == NULL)
+	return "";
+
     switch (fstype) {
 
     case 0:
@@ -82,81 +86,53 @@ bsd_get_desc(uint8_t fstype)
 /* 
  * Process the partition table at the sector address 
  *
+ * Return 1 on error and 0 if no error
  */
 static uint8_t
-bsd_load_table(MM_INFO * mm, uint8_t test)
+bsd_load_table(MM_INFO * mm)
 {
 
     bsd_disklabel dlabel;
     uint32_t idx = 0;
+    SSIZE_T cnt;
+    DADDR_T laddr = mm->offset / mm->block_size + BSD_PART_SOFFSET;	// used for printing only
+    DADDR_T max_addr = (mm->img_info->size - mm->offset) / mm->block_size;	// max sector
 
     if (verbose)
 	fprintf(stderr, "bsd_load_table: Table Sector: %" PRIuDADDR "\n",
-		mm->sect_offset);
+	    laddr);
 
-    /* seek to the table */
-    if (mm_read_block_nobuf
-	(mm, (char *) &dlabel, sizeof(dlabel),
-	 mm->sect_offset) != sizeof(dlabel)) {
-	if (test) {
-	    fprintf(stderr,
-		    "Error reading BSD Disk Label in Sector: %" PRIuDADDR
-		    "\n", mm->sect_offset);
-	    return 1;
-	}
-	else {
-	    error("Error reading BSD Disk Label in Sector: %" PRIuDADDR
-		  "\n", mm->sect_offset);
+    /* read the block */
+    cnt = mm_read_block_nobuf
+	(mm, (char *) &dlabel, sizeof(dlabel), BSD_PART_SOFFSET);
+    if (cnt != sizeof(dlabel)) {
+	snprintf(tsk_errstr2, TSK_ERRSTR_L,
+	    "BSD Disk Label in Sector: %" PRIuDADDR, laddr);
+	if (cnt != -1) {
+	    tsk_errno = TSK_ERR_MM_READ;
+	    tsk_errstr[0] = '\0';
 	}
+	return 1;
     }
 
     /* Check the magic  */
-    if (guessu32(mm, dlabel.magic, BSD_MAGIC)) {
-
-	/* Check if the next sector has the correct magic value because
-	 * the user may have given the first partition sector, but the
-	 * dlabel is in the second sector
-	 */
-	mm->sect_offset++;
-	if (mm_read_block_nobuf
-	    (mm, (char *) &dlabel, sizeof(dlabel),
-	     mm->sect_offset) != sizeof(dlabel)) {
-
-	    if (test) {
-		fprintf(stderr,
-			"Error reading BSD Disk Label (second try) in Sector: %"
-			PRIuDADDR "\n", mm->sect_offset);
-		return 1;
-	    }
-	    else {
-		error
-		    ("Error reading BSD Disk Label (second try)in Sector: %"
-		     PRIuDADDR "\n", mm->sect_offset);
-	    }
-	}
-
-
-	/* If it isn't there, then give an error */
-	if (guessu32(mm, dlabel.magic, BSD_MAGIC)) {
-	    if (test)
-		return 1;
-	    else
-		error
-		    ("Invalid BSD partition table (magic #1) (Sector: %"
-		     PRIuDADDR ") %" PRIx32 "", mm->sect_offset - 1,
-		     getu32(mm, dlabel.magic));
-	}
+    if (mm_guessu32(mm, dlabel.magic, BSD_MAGIC)) {
+	tsk_errno = TSK_ERR_MM_MAGIC;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "BSD partition table (magic #1) (Sector: %"
+	    PRIuDADDR ") %" PRIx32, laddr, getu32(mm, dlabel.magic));
+	tsk_errstr2[0] = '\0';
+	return 1;
     }
 
     /* Check the second magic value */
     if (getu32(mm, dlabel.magic2) != BSD_MAGIC) {
-	if (test)
-	    return 1;
-	else
-	    error("Invalid BSD disk label (magic #2) (Sector: %"
-		  PRIuDADDR ")  %" PRIx32 "\n", mm->sect_offset, getu32(mm,
-									dlabel.
-									magic2));
+	tsk_errno = TSK_ERR_MM_MAGIC;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "BSD disk label (magic #2) (Sector: %"
+	    PRIuDADDR ")  %" PRIx32, laddr, getu32(mm, dlabel.magic2));
+	tsk_errstr2[0] = '\0';
+	return 1;
     }
 
     /* Cycle through the partitions, there are either 8 or 16 */
@@ -170,17 +146,28 @@ bsd_load_table(MM_INFO * mm, uint8_t test)
 
 	if (verbose)
 	    fprintf(stderr,
-		    "load_table: %d  Starting Sector: %" PRIu32
-		    "  Size: %" PRIu32 "  Type: %d\n", idx, part_start,
-		    part_size, dlabel.part[idx].fstype);
+		"load_table: %d  Starting Sector: %" PRIu32
+		"  Size: %" PRIu32 "  Type: %d\n", idx, part_start,
+		part_size, dlabel.part[idx].fstype);
 
 	if (part_size == 0)
 	    continue;
 
+	if (part_start > max_addr) {
+	    tsk_errno = TSK_ERR_MM_BLK_NUM;
+	    snprintf(tsk_errstr, TSK_ERRSTR_L,
+		"bsd_load_table: Starting sector too large for image");
+	    tsk_errstr2[0] = '\0';
+	    return 1;
+	}
+
+
 	/* Add the partition to the internal sorted list */
-	mm_part_add(mm, (DADDR_T) part_start, (DADDR_T) part_size,
-		    MM_TYPE_VOL,
-		    bsd_get_desc(dlabel.part[idx].fstype), -1, idx);
+	if (NULL == mm_part_add(mm, (DADDR_T) part_start,
+		(DADDR_T) part_size, MM_TYPE_VOL,
+		bsd_get_desc(dlabel.part[idx].fstype), -1, idx)) {
+	    return 1;
+	}
     }
 
     return 0;
@@ -190,32 +177,50 @@ bsd_load_table(MM_INFO * mm, uint8_t test)
 /* 
  * Walk the partitions that have already been loaded during _open
  *
+ * Return 1 on error and 0 on success
  */
-void
+uint8_t
 bsd_part_walk(MM_INFO * mm, PNUM_T start, PNUM_T last, int flags,
-	      MM_PART_WALK_FN action, char *ptr)
+    MM_PART_WALK_FN action, char *ptr)
 {
     MM_PART *part;
     PNUM_T cnt = 0;
 
-    if (start < mm->first_part || start > mm->last_part)
-	error("bsd_part_walk: Invalid start partition: %" PRIuPNUM "",
-	      start);
+    if (start < mm->first_part || start > mm->last_part) {
+	tsk_errno = TSK_ERR_MM_WALK_RNG;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "bsd_part_walk: Start partition: %" PRIuPNUM "", start);
+	tsk_errstr2[0] = '\0';
+	return 1;
+    }
 
-    if (last < mm->first_part || last > mm->last_part)
-	error("bsd_part_walk: Invalid end partition: %" PRIuPNUM "", last);
+    if (last < mm->first_part || last > mm->last_part) {
+	tsk_errno = TSK_ERR_MM_WALK_RNG;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "bsd_part_walk: End partition: %" PRIuPNUM "", last);
+	tsk_errstr2[0] = '\0';
+	return 1;
+    }
 
     part = mm->part_list;
     while ((part != NULL) && (cnt <= last)) {
 
-	if (cnt >= start)
-	    action(mm, cnt, part, 0, ptr);
+	if (cnt >= start) {
+	    int retval;
+	    retval = action(mm, cnt, part, 0, ptr);
+	    if (retval == WALK_STOP) {
+		return 0;
+	    }
+	    else if (retval == WALK_ERROR) {
+		return 1;
+	    }
+	}
 
 	part = part->next;
 	cnt++;
     }
 
-    return;
+    return 0;
 }
 
 
@@ -226,18 +231,25 @@ bsd_close(MM_INFO * mm)
     free(mm);
 }
 
+/*
+ * analyze the image in img_info and process it as BSD
+ * Initialize the MM_INFO structure
+ *
+ * Return MM_INFO or NULL if not BSD or an error 
+ */
 MM_INFO *
-bsd_open(IMG_INFO * img_info, DADDR_T offset, uint8_t test)
+bsd_open(IMG_INFO * img_info, DADDR_T offset)
 {
     MM_INFO *mm = (MM_INFO *) mymalloc(sizeof(*mm));
+    if (mm == NULL)
+	return NULL;
 
     mm->img_info = img_info;
     mm->mmtype = MM_BSD;
     mm->str_type = "BSD Disk Label";
 
-
     /* use the offset provided */
-    mm->sect_offset = offset + BSD_PART_OFFSET;
+    mm->offset = offset;
 
     /* inititialize settings */
     mm->part_list = NULL;
@@ -251,13 +263,16 @@ bsd_open(IMG_INFO * img_info, DADDR_T offset, uint8_t test)
     mm->close = bsd_close;
 
     /* Load the partitions into the sorted list */
-    if (bsd_load_table(mm, test)) {
+    if (bsd_load_table(mm)) {
 	bsd_close(mm);
 	return NULL;
     }
 
     /* fill in the sorted list with the 'unknown' values */
-    mm_part_unused(mm);
+    if (mm_part_unused(mm)) {
+	bsd_close(mm);
+	return NULL;
+    }
 
     return mm;
 }
