@@ -1,9 +1,11 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2005/09/02 23:34:04 $
+ * $Date: 2006/04/07 21:21:03 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
+ * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
+ *
  * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
  *
  * DOS Partition Tables
@@ -33,6 +35,8 @@ dos_get_desc(uint8_t ptype)
 {
 #define DESC_LEN 64
     char *str = mymalloc(DESC_LEN);
+    if (str == NULL)
+	return "";
 
     switch (ptype) {
     case 0x00:
@@ -181,8 +185,7 @@ dos_get_desc(uint8_t ptype)
     case 0xc6:
     case 0xc7:
 	snprintf(str, DESC_LEN,
-		 "Corrupted Windows NT Volume / Stripe Set (0x%.2X)",
-		 ptype);
+	    "Corrupted Windows NT Volume / Stripe Set (0x%.2X)", ptype);
 	break;
     case 0xee:
 	snprintf(str, DESC_LEN, "GPT Safety Partition (0xEE)");
@@ -282,7 +285,7 @@ dos_get_desc(uint8_t ptype)
 	break;
     case 0x41:
 	snprintf(str, DESC_LEN,
-		 "Linux/MINIX (Sharing Disk with DR-DOS) (0x41)");
+	    "Linux/MINIX (Sharing Disk with DR-DOS) (0x41)");
 	break;
 	/*      
 	   case 0x41:
@@ -294,7 +297,7 @@ dos_get_desc(uint8_t ptype)
 	 */
     case 0x43:
 	snprintf(str, DESC_LEN,
-		 "Linux Native (Sharing Disk with DR-DOS) (0x43)");
+	    "Linux Native (Sharing Disk with DR-DOS) (0x43)");
 	break;
     case 0x45:
 	snprintf(str, DESC_LEN, "Boot-US Boot Manager (0x45)");
@@ -318,7 +321,7 @@ dos_get_desc(uint8_t ptype)
 	break;
     case 0x4a:
 	snprintf(str, DESC_LEN,
-		 "Mark Aitchison's ALFS/THIN Lightweight Filesystem (0x4a)");
+	    "Mark Aitchison's ALFS/THIN Lightweight Filesystem (0x4a)");
 	break;
 	/*case 0x4a:
 	   snprintf(str, DESC_LEN, "AdaOS Aquila (0x4a)");
@@ -348,7 +351,7 @@ dos_get_desc(uint8_t ptype)
 	   break; */
     case 0x56:
 	snprintf(str, DESC_LEN,
-		 "AT&T MS-DOS 3.x Logically Sectored FAT. (0x56)");
+	    "AT&T MS-DOS 3.x Logically Sectored FAT. (0x56)");
 	break;
     case 0x5c:
 	snprintf(str, DESC_LEN, "Priam EDisk (0x5c)");
@@ -391,22 +394,22 @@ dos_get_desc(uint8_t ptype)
 	break;
     case 0x8c:
 	snprintf(str, DESC_LEN,
-		 "Legacy Fault Tolerant FAT32 using BIOS extd INT 13h (0x8c)");
+	    "Legacy Fault Tolerant FAT32 using BIOS extd INT 13h (0x8c)");
 	break;
     case 0x8d:
 	snprintf(str, DESC_LEN,
-		 "Free FDISK Hidden Primary DOS FAT12 (0x8d)");
+	    "Free FDISK Hidden Primary DOS FAT12 (0x8d)");
 	break;
     case 0x90:
 	snprintf(str, DESC_LEN,
-		 "Free FDISK Hidden Primary DOS FAT16 (0x90)");
+	    "Free FDISK Hidden Primary DOS FAT16 (0x90)");
 	break;
     case 0x91:
 	snprintf(str, DESC_LEN, "Free FDISK Hidden DOS Extended (0x91)");
 	break;
     case 0x92:
 	snprintf(str, DESC_LEN,
-		 "Free FDISK Hidden Primary DOS Large FAT16 (0x92)");
+	    "Free FDISK Hidden Primary DOS Large FAT16 (0x92)");
 	break;
     case 0x94:
 	snprintf(str, DESC_LEN, "Amoeba Bad Block Table (0x94)");
@@ -416,11 +419,11 @@ dos_get_desc(uint8_t ptype)
 	break;
     case 0x97:
 	snprintf(str, DESC_LEN,
-		 "Free FDISK Hidden Primary DOS FAT32 (0x97)");
+	    "Free FDISK Hidden Primary DOS FAT32 (0x97)");
 	break;
     case 0x98:
 	snprintf(str, DESC_LEN,
-		 "Free FDISK Hidden Primary DOS FAT32 LBA (0x98)");
+	    "Free FDISK Hidden Primary DOS FAT32 LBA (0x98)");
 	break;
 	/*case 0x98:
 	   snprintf(str, DESC_LEN, "Datalight ROM-DOS Super-Boot (0x98)");
@@ -430,19 +433,19 @@ dos_get_desc(uint8_t ptype)
 	break;
     case 0x9a:
 	snprintf(str, DESC_LEN,
-		 "Free FDISK Hidden Primary DOS FAT16 LBA (0x9a)");
+	    "Free FDISK Hidden Primary DOS FAT16 LBA (0x9a)");
 	break;
     case 0x9b:
 	snprintf(str, DESC_LEN,
-		 "Free FDISK Hidden DOS Extended LBA (0x9b)");
+	    "Free FDISK Hidden DOS Extended LBA (0x9b)");
 	break;
     case 0xa3:
 	snprintf(str, DESC_LEN,
-		 "HP Volume Expansion (SpeedStor Variant) (0xa3)");
+	    "HP Volume Expansion (SpeedStor Variant) (0xa3)");
 	break;
     case 0xa4:
 	snprintf(str, DESC_LEN,
-		 "HP Volume Expansion (SpeedStor Variant) (0xa4)");
+	    "HP Volume Expansion (SpeedStor Variant) (0xa4)");
 	break;
     case 0xaa:
 	snprintf(str, DESC_LEN, "Olivetti Fat 12 1.44MB Service (0xaa)");
@@ -458,22 +461,22 @@ dos_get_desc(uint8_t ptype)
 	break;
     case 0xb1:
 	snprintf(str, DESC_LEN,
-		 "HP Volume Expansion (SpeedStor Variant) (0xb1)");
+	    "HP Volume Expansion (SpeedStor Variant) (0xb1)");
 	break;
     case 0xb3:
 	snprintf(str, DESC_LEN,
-		 "HP Volume Expansion (SpeedStor Variant) (0xb3)");
+	    "HP Volume Expansion (SpeedStor Variant) (0xb3)");
 	break;
     case 0xb4:
 	snprintf(str, DESC_LEN,
-		 "HP Volume Expansion (SpeedStor Variant) (0xb4)");
+	    "HP Volume Expansion (SpeedStor Variant) (0xb4)");
 	break;
 	/*case 0xb6:
 	   snprintf(str, DESC_LEN, "HP Volume Expansion (SpeedStor Variant) (0xb6)");
 	   break; */
     case 0xb6:
 	snprintf(str, DESC_LEN,
-		 "Corrupted Windows NT Mirror Set Master FAT16 (0xb6)");
+	    "Corrupted Windows NT Mirror Set Master FAT16 (0xb6)");
 	break;
     case 0xbb:
 	snprintf(str, DESC_LEN, "Boot Wizard Hidden (0xbb)");
@@ -534,15 +537,15 @@ dos_get_desc(uint8_t ptype)
 	break;
     case 0xd4:
 	snprintf(str, DESC_LEN,
-		 "Old Multiuser DOS Secured FAT16 <32M (0xd4)");
+	    "Old Multiuser DOS Secured FAT16 <32M (0xd4)");
 	break;
     case 0xd5:
 	snprintf(str, DESC_LEN,
-		 "Old Multiuser DOS Secured extended (0xd5)");
+	    "Old Multiuser DOS Secured extended (0xd5)");
 	break;
     case 0xd6:
 	snprintf(str, DESC_LEN,
-		 "Old Multiuser DOS Secured FAT16 >=32M (0xd6)");
+	    "Old Multiuser DOS Secured FAT16 >=32M (0xd6)");
 	break;
     case 0xd8:
 	snprintf(str, DESC_LEN, "CP/M-86 (0xd8)");
@@ -552,7 +555,7 @@ dos_get_desc(uint8_t ptype)
 	break;
     case 0xdb:
 	snprintf(str, DESC_LEN,
-		 "Digital Research CP/M, Concurrent CP/M, Concurrent DOS (0xdb)");
+	    "Digital Research CP/M, Concurrent CP/M, Concurrent DOS (0xdb)");
 	break;
 	/*case 0xdb:
 	   snprintf(str, DESC_LEN, "Unisys CTOS (0xdb)");
@@ -574,22 +577,22 @@ dos_get_desc(uint8_t ptype)
 	   break; */
     case 0xe0:
 	snprintf(str, DESC_LEN,
-		 "Reserved by STMicroelectronics for ST AVFS. (0xe0)");
+	    "Reserved by STMicroelectronics for ST AVFS. (0xe0)");
 	break;
     case 0xe1:
 	snprintf(str, DESC_LEN,
-		 "DOS Access or SpeedStor 12-bit FAT Extended (0xe1)");
+	    "DOS Access or SpeedStor 12-bit FAT Extended (0xe1)");
 	break;
     case 0xe3:
 	snprintf(str, DESC_LEN, "DOS R/O or SpeedStor (0xe3)");
 	break;
     case 0xe4:
 	snprintf(str, DESC_LEN,
-		 "SpeedStor 16-bit FAT Extended <1024 cyl. (0xe4)");
+	    "SpeedStor 16-bit FAT Extended <1024 cyl. (0xe4)");
 	break;
     case 0xe5:
 	snprintf(str, DESC_LEN,
-		 "Tandy MS-DOS with Logically Sectored FAT (0xe5)");
+	    "Tandy MS-DOS with Logically Sectored FAT (0xe5)");
 	break;
     case 0xe6:
 	snprintf(str, DESC_LEN, "Storage Dimensions SpeedStor (0xe6)");
@@ -638,7 +641,7 @@ dos_get_desc(uint8_t ptype)
 	   break; */
     case 0xfe:
 	snprintf(str, DESC_LEN,
-		 "Windows NT Disk Administrator Hidden (0xfe)");
+	    "Windows NT Disk Administrator Hidden (0xfe)");
 	break;
 	/*case 0xfe:
 	   snprintf(str, DESC_LEN, "Old Linux Logical Volume Manager (0xfe)");
@@ -665,51 +668,57 @@ dos_get_desc(uint8_t ptype)
  * 
  * For the primary extended table, sect_cur == sect_ext_base 
  *
+ * Return 1 on error and 0 on success
+ *
  */
 static uint8_t
 dos_load_ext_table(MM_INFO * mm, DADDR_T sect_cur, DADDR_T sect_ext_base,
-		   int table, uint8_t test)
+    int table)
 {
     dos_sect sect;
     int i;
     char *table_str;
+    SSIZE_T cnt;
+    DADDR_T max_addr = (mm->img_info->size - mm->offset) / mm->block_size;	// max sector
 
     if (verbose)
 	fprintf(stderr,
-		"dos_load_ext: Table Sector: %" PRIuDADDR
-		", Primary Base Sector: %" PRIuDADDR "\n", sect_cur,
-		sect_ext_base);
+	    "dos_load_ext: Table Sector: %" PRIuDADDR
+	    ", Primary Base Sector: %" PRIuDADDR "\n", sect_cur,
+	    sect_ext_base);
 
     /* Read the partition table sector */
-    if (mm_read_block_nobuf
-	(mm, (char *) &sect, sizeof(sect),
-	 (DADDR_T) sect_cur) != sizeof(sect)) {
-	if (test) {
-	    fprintf(stderr,
-		    "Error reading extended table sector %" PRIuDADDR "\n",
-		    sect_cur);
-	    return 1;
-	}
-	else {
-	    error("Error reading extended table sector %" PRIuDADDR "\n",
-		  sect_cur);
+    cnt = mm_read_block_nobuf
+	(mm, (char *) &sect, sizeof(sect), (DADDR_T) sect_cur);
+    if (cnt != sizeof(sect)) {
+	snprintf(tsk_errstr2, TSK_ERRSTR_L,
+	    "Extended DOS table sector %" PRIuDADDR, sect_cur);
+	if (cnt != -1) {
+	    tsk_errno = TSK_ERR_MM_READ;
+	    tsk_errstr[0] = '\0';
 	}
+	return 1;
     }
 
     /* Sanity Check */
     if (getu16(mm, sect.magic) != DOS_MAGIC) {
-	if (test)
-	    return 1;
-	else
-	    error("Invalid extended partition table magic in sector %"
-		  PRIuDADDR "\n", sect_cur);
+	tsk_errno = TSK_ERR_MM_MAGIC;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "Extended DOS partition table in sector %"
+	    PRIuDADDR, sect_cur);
+	tsk_errstr2[0] = '\0';
+	return 1;
     }
 
     /* Add an entry of 1 length for the table  to the internal structure */
-    table_str = mymalloc(32);
+    if ((table_str = mymalloc(32)) == NULL)
+	return 1;
+
     snprintf(table_str, 32, "Extended Table (#%d)", table);
-    mm_part_add(mm, (DADDR_T) sect_cur, (DADDR_T) 1, MM_TYPE_DESC,
-		table_str, table, -1);
+    if (NULL == mm_part_add(mm, (DADDR_T) sect_cur, (DADDR_T) 1,
+	    MM_TYPE_DESC, table_str, table, -1)) {
+	return 1;
+    }
 
     /* Cycle through the four partitions in the table 
      *
@@ -726,9 +735,9 @@ dos_load_ext_table(MM_INFO * mm, DADDR_T sect_cur, DADDR_T sect_ext_base,
 
 	if (verbose)
 	    fprintf(stderr,
-		    "load_ext: %d:%d    Start: %" PRIu32 "   Size: %"
-		    PRIu32 "  Type: %d\n", table, i, part_start, part_size,
-		    part->ptype);
+		"load_ext: %d:%d    Start: %" PRIu32 "   Size: %"
+		PRIu32 "  Type: %d\n", table, i, part_start, part_size,
+		part->ptype);
 
 	if (part_size == 0)
 	    continue;
@@ -740,14 +749,25 @@ dos_load_ext_table(MM_INFO * mm, DADDR_T sect_cur, DADDR_T sect_ext_base,
 	    /* part start is added to the start of the
 	     * first extended partition (the primary
 	     * extended partition) */
-	    mm_part_add(mm, (DADDR_T) (sect_ext_base + part_start),
-			(DADDR_T) part_size, MM_TYPE_DESC,
-			dos_get_desc(part->ptype), table, i);
+
+	    if (sect_ext_base + part_start > max_addr) {
+		tsk_errno = TSK_ERR_MM_BLK_NUM;
+		snprintf(tsk_errstr, TSK_ERRSTR_L,
+		    "dos_load_ext_table: Starting sector too large for image");
+		tsk_errstr2[0] = '\0';
+		return 1;
+	    }
+
+	    if (NULL == mm_part_add(mm,
+		    (DADDR_T) (sect_ext_base + part_start),
+		    (DADDR_T) part_size, MM_TYPE_DESC,
+		    dos_get_desc(part->ptype), table, i))
+		return 1;
 
 
 	    /* Process the extended partition */
 	    if (dos_load_ext_table(mm, sect_ext_base + part_start,
-				   sect_ext_base, table + 1, test))
+		    sect_ext_base, table + 1))
 		return 1;
 	}
 
@@ -755,9 +775,18 @@ dos_load_ext_table(MM_INFO * mm, DADDR_T sect_cur, DADDR_T sect_ext_base,
 	    /* part_start is added to the start of the 
 	     * current partition for the actual
 	     * starting location */
-	    mm_part_add(mm, (DADDR_T) (sect_cur + part_start),
-			(DADDR_T) part_size, MM_TYPE_VOL,
-			dos_get_desc(part->ptype), table, i);
+
+	    if (sect_cur + part_start > max_addr) {
+		tsk_errno = TSK_ERR_MM_BLK_NUM;
+		snprintf(tsk_errstr, TSK_ERRSTR_L,
+		    "dos_load_ext_table: Starting sector too large for image");
+		tsk_errstr2[0] = '\0';
+		return 1;
+	    }
+	    if (NULL == mm_part_add(mm, (DADDR_T) (sect_cur + part_start),
+		    (DADDR_T) part_size, MM_TYPE_VOL,
+		    dos_get_desc(part->ptype), table, i))
+		return 1;
 	}
     }
 
@@ -774,9 +803,7 @@ dos_load_ext_table(MM_INFO * mm, DADDR_T sect_cur, DADDR_T sect_ext_base,
  *
  * sect_cur is the addres of the table to load
  *
- * test is set to 1 when testing if this is DOS and return 1 if it is not
- *
- * 0 is returned if the load is successful
+ * 0 is returned if the load is successful and 1 if error
  */
 static uint8_t
 dos_load_prim_table(MM_INFO * mm, uint8_t test)
@@ -784,37 +811,37 @@ dos_load_prim_table(MM_INFO * mm, uint8_t test)
     dos_sect sect;
     int i;
     char *table_str;
+    SSIZE_T cnt;
+    DADDR_T taddr = mm->offset / mm->block_size + DOS_PART_SOFFSET;
+    DADDR_T max_addr = (mm->img_info->size - mm->offset) / mm->block_size;	// max sector
 
     if (verbose)
 	fprintf(stderr,
-		"dos_load_prim: Table Sector: %" PRIuDADDR "\n",
-		mm->sect_offset);
+	    "dos_load_prim: Table Sector: %" PRIuDADDR "\n", taddr);
 
     /* Read the table */
-    if (mm_read_block_nobuf
-	(mm, (char *) &sect, sizeof(sect),
-	 (DADDR_T) mm->sect_offset) != sizeof(sect)) {
-	if (test) {
-	    fprintf(stderr,
-		    "Error reading primary table sector %" PRIuDADDR "\n",
-		    mm->sect_offset);
-	    return 1;
-	}
-	else {
-	    error("Error reading primary table sector %" PRIuDADDR "\n",
-		  mm->sect_offset);
+    cnt = mm_read_block_nobuf
+	(mm, (char *) &sect, sizeof(sect), DOS_PART_SOFFSET);
+
+    if (cnt != sizeof(sect)) {
+	snprintf(tsk_errstr2, TSK_ERRSTR_L,
+	    "Primary DOS table sector %" PRIuDADDR, taddr);
+	if (cnt != -1) {
+	    tsk_errno = TSK_ERR_MM_READ;
+	    tsk_errstr[0] = '\0';
 	}
+	return 1;
     }
 
 
     /* Sanity Check */
-    if (guessu16(mm, sect.magic, DOS_MAGIC)) {
-	if (test)
-	    return 1;
-	else
-	    error
-		("File is not a DOS partition (invalid primary magic) (Sector: %"
-		 PRIuDADDR ")\n", mm->sect_offset);
+    if (mm_guessu16(mm, sect.magic, DOS_MAGIC)) {
+	tsk_errno = TSK_ERR_MM_MAGIC;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "File is not a DOS partition (invalid primary magic) (Sector: %"
+	    PRIuDADDR ")", taddr);
+	tsk_errstr2[0] = '\0';
+	return 1;
     }
 
     /* Because FAT and NTFS use the same magic - check for a
@@ -825,33 +852,39 @@ dos_load_prim_table(MM_INFO * mm, uint8_t test)
 
 	if (verbose)
 	    fprintf(stderr,
-		    "dos_load_prim_table: Testing FAT/NTFS conditions\n");
+		"dos_load_prim_table: Testing FAT/NTFS conditions\n");
 
 	if (strncmp("MSDOS", sect.oemname, 5) == 0) {
-	    if (verbose)
-		fprintf(stderr,
-			"dos_load_prim_table: Test fail - MSDOS OEM name\n");
+	    tsk_errno = TSK_ERR_MM_MAGIC;
+	    snprintf(tsk_errstr, TSK_ERRSTR_L,
+		"dos_load_prim_table: MSDOS OEM name exists");
+	    tsk_errstr2[0] = '\0';
 	    return 1;
 	}
 	else if (strncmp("MSWIN", sect.oemname, 5) == 0) {
-	    if (verbose)
-		fprintf(stderr,
-			"dos_load_prim_table: Test fail - MSWIN OEM name\n");
+	    tsk_errno = TSK_ERR_MM_MAGIC;
+	    snprintf(tsk_errstr, TSK_ERRSTR_L,
+		"dos_load_prim_table: MSWIN OEM name exists");
+	    tsk_errstr2[0] = '\0';
 	    return 1;
 	}
 	else if (strncmp("NTFS", sect.oemname, 4) == 0) {
-	    if (verbose)
-		fprintf(stderr,
-			"dos_load_prim_table: Test fail - NTFS OEM name\n");
+	    tsk_errno = TSK_ERR_MM_MAGIC;
+	    snprintf(tsk_errstr, TSK_ERRSTR_L,
+		"dos_load_prim_table: NTFS OEM name exists");
+	    tsk_errstr2[0] = '\0';
 	    return 1;
 	}
     }
 
     /* Add an entry of 1 sector for the table  to the internal structure */
-    table_str = mymalloc(32);
+    if ((table_str = mymalloc(32)) == NULL)
+	return 1;
+
     snprintf(table_str, 32, "Primary Table (#0)");
-    mm_part_add(mm, (DADDR_T) mm->sect_offset, (DADDR_T) 1, MM_TYPE_DESC,
-		table_str, -1, -1);
+    if (NULL == mm_part_add(mm, DOS_PART_SOFFSET, (DADDR_T) 1,
+	    MM_TYPE_DESC, table_str, -1, -1))
+	return 1;
 
     /* Cycle through the partition table */
     for (i = 0; i < 4; i++) {
@@ -863,24 +896,46 @@ dos_load_prim_table(MM_INFO * mm, uint8_t test)
 
 	if (verbose)
 	    fprintf(stderr,
-		    "load_pri:0:%d    Start: %" PRIu32 "   Size: %" PRIu32
-		    "  Type: %d\n", i, part_start, part_size, part->ptype);
+		"load_pri:0:%d    Start: %" PRIu32 "   Size: %" PRIu32
+		"  Type: %d\n", i, part_start, part_size, part->ptype);
 
 	if (part_size == 0)
 	    continue;
 
+	if (part_start > max_addr) {
+	    tsk_errno = TSK_ERR_MM_BLK_NUM;
+	    snprintf(tsk_errstr, TSK_ERRSTR_L,
+		"dos_load_prim_table: Starting sector too large for image");
+	    tsk_errstr2[0] = '\0';
+	    return 1;
+	}
+#if 0
+// I'm not sure if this is too strict ...
+	else if ((part_start + part_size) > max_addr) {
+	    tsk_errno = TSK_ERR_MM_BLK_NUM
+		snprintf(tsk_errstr, TSK_ERRSTR_L,
+		"dos_load_prim_table: Partition ends after image");
+	    tsk_errstr2[0] = '\0';
+	    return 1;
+	}
+#endif
+
 	/* Add the partition to the internal structure 
 	 * If it is an extended partition, process it now */
 	if (dos_is_ext(part->ptype)) {
-	    mm_part_add(mm, (DADDR_T) part_start, (DADDR_T) part_size,
-			MM_TYPE_DESC, dos_get_desc(part->ptype), 0, i);
+	    if (NULL == mm_part_add(mm, (DADDR_T) part_start,
+		    (DADDR_T) part_size, MM_TYPE_DESC,
+		    dos_get_desc(part->ptype), 0, i))
+		return 1;
 
-	    if (dos_load_ext_table(mm, part_start, part_start, 1, test))
+	    if (dos_load_ext_table(mm, part_start, part_start, 1))
 		return 1;
 	}
 	else {
-	    mm_part_add(mm, (DADDR_T) part_start, (DADDR_T) part_size,
-			MM_TYPE_VOL, dos_get_desc(part->ptype), 0, i);
+	    if (NULL == mm_part_add(mm, (DADDR_T) part_start,
+		    (DADDR_T) part_size, MM_TYPE_VOL,
+		    dos_get_desc(part->ptype), 0, i))
+		return 1;
 	}
     }
     return 0;
@@ -890,34 +945,49 @@ dos_load_prim_table(MM_INFO * mm, uint8_t test)
 /* 
  * Walk the partitions that have already been loaded during _open
  *
+ * Return 1 on error and 0 on success
  */
-void
+uint8_t
 dos_part_walk(MM_INFO * mm, PNUM_T start, PNUM_T last, int flags,
-	      MM_PART_WALK_FN action, char *ptr)
+    MM_PART_WALK_FN action, char *ptr)
 {
     MM_PART *part;
     PNUM_T cnt = 0;
 
-    if (start < mm->first_part || start > mm->last_part)
-	error("dos_part_walk: Invalid starting partition: %" PRIuPNUM "",
-	      start);
-    if (last < mm->first_part || last > mm->last_part)
-	error("dos_part_walk: Invalid ending partition: %" PRIuPNUM "",
-	      last);
+    if (start < mm->first_part || start > mm->last_part) {
+	tsk_errno = TSK_ERR_MM_WALK_RNG;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "dos_part_walk: Starting partition: %" PRIuPNUM "", start);
+	tsk_errstr2[0] = '\0';
+	return 1;
+    }
+    if (last < mm->first_part || last > mm->last_part) {
+	tsk_errno = TSK_ERR_MM_WALK_RNG;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "dos_part_walk: Ending partition: %" PRIuPNUM "", last);
+	tsk_errstr2[0] = '\0';
+	return 1;
+    }
 
     /* Cycle through the partitions */
     part = mm->part_list;
     while ((part != NULL) && (cnt <= last)) {
 
 	/* Can we do the action yet? */
-	if (cnt >= start)
-	    action(mm, cnt, part, 0, ptr);
+	if (cnt >= start) {
+	    int retval;
+	    retval = action(mm, cnt, part, 0, ptr);
+	    if (retval == WALK_STOP)
+		return 0;
+	    else if (retval == WALK_ERROR)
+		return 1;
+	}
 
 	part = part->next;
 	cnt++;
     }
 
-    return;
+    return 0;
 }
 
 
@@ -933,20 +1003,23 @@ dos_close(MM_INFO * mm)
  * Given the path to the file, open it and load the internal
  * partition table structure
  *
- * offset is the sector offset to the begiinning of the structure
- * (which maynot be the beginning of a partition)
+ * offset is the byte offset to the start of the volume system
+ *
+ * If test is 1 then additional tests are performed to make sure 
+ * it isn't a FAT or NTFS file system
  */
 MM_INFO *
 dos_open(IMG_INFO * img_info, DADDR_T offset, uint8_t test)
 {
     MM_INFO *mm = (MM_INFO *) mymalloc(sizeof(*mm));
+    if (mm == NULL)
+	return NULL;
 
     mm->mmtype = MM_DOS;
     mm->str_type = "DOS Partition Table";
     mm->img_info = img_info;
 
-    /* If an offset was given, then use that too */
-    mm->sect_offset = offset + DOS_PART_OFFSET;
+    mm->offset = offset;
 
     /* inititialize settings */
     mm->part_list = NULL;
@@ -967,7 +1040,10 @@ dos_open(IMG_INFO * img_info, DADDR_T offset, uint8_t test)
     }
 
     /* fill in the sorted list with the 'unknown' values */
-    mm_part_unused(mm);
+    if (mm_part_unused(mm)) {
+	dos_close(mm);
+	return NULL;
+    }
 
     return mm;
 }
