@@ -1,9 +1,10 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2005/09/02 23:34:04 $
+ * $Date: 2006/05/11 15:48:12 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
+ * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
  * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
  *
  * mmls - list media management structure contents
@@ -28,18 +29,17 @@ void
 usage()
 {
     fprintf(stderr,
-	    "%s [-i imgtype] [-o imgoffset] [-brvV] [-t mmtype] image [images]\n",
-	    progname);
+	"%s [-i imgtype] [-o imgoffset] [-brvV] [-t mmtype] image [images]\n",
+	progname);
     fprintf(stderr, "\t-t mmtype: The type of partition system\n");
     fprintf(stderr, "\t-i imgtype: The format of the image file\n");
     fprintf(stderr,
-	    "\t-o imgoffset: Offset to the start of the volume that contains the partition system (in sectors)\n");
+	"\t-o imgoffset: Offset to the start of the volume that contains the partition system (in sectors)\n");
     fprintf(stderr, "\t-b: print the rounded length in bytes\n");
     fprintf(stderr,
-	    "\t-r: recurse and look for other partition tables in partitions (DOS Only)\n");
+	"\t-r: recurse and look for other partition tables in partitions (DOS Only)\n");
     fprintf(stderr, "\t-v: verbose output\n");
     fprintf(stderr, "\t-V: print the version\n");
-    fprintf(stderr, "Supported partition systems:\n");
     mm_print_types(stderr);
     fprintf(stderr, "Supported image formats:\n");
     img_print_types(stderr);
@@ -48,6 +48,8 @@ usage()
 
 /*
  * The callback action for the part_walk
+ *
+ * Prints the layout information
  * */
 uint8_t
 part_act(MM_INFO * mm, PNUM_T pnum, MM_PART * part, int flag, char *ptr)
@@ -59,7 +61,7 @@ part_act(MM_INFO * mm, PNUM_T pnum, MM_PART * part, int flag, char *ptr)
     /* Table was not given, but slot was */
     else if ((part->table_num == -1) && (part->slot_num != -1))
 	printf("%.2" PRIuPNUM ":  %.2" PRIuPNUM "      ",
-	       pnum, part->slot_num);
+	    pnum, part->slot_num);
 
     /* The Table was given, but slot wasn't */
     else if ((part->table_num != -1) && (part->slot_num == -1))
@@ -68,7 +70,7 @@ part_act(MM_INFO * mm, PNUM_T pnum, MM_PART * part, int flag, char *ptr)
     /* Both table and slot were given */
     else if ((part->table_num != -1) && (part->slot_num != -1))
 	printf("%.2" PRIuPNUM ":  %.2d:%.2d   ",
-	       pnum, part->table_num, part->slot_num);
+	    pnum, part->table_num, part->slot_num);
 
     if (print_bytes) {
 	OFF_T size;
@@ -98,21 +100,23 @@ part_act(MM_INFO * mm, PNUM_T pnum, MM_PART * part, int flag, char *ptr)
 
 	/* Print the layout */
 	printf("%.10" PRIuDADDR "   %.10" PRIuDADDR "   %.10" PRIuDADDR
-	       "   %.4" PRIuOFF "%c   %s\n", part->start,
-	       (part->start + part->len - 1), part->len, size, unit,
-	       part->desc);
+	    "   %.4" PRIuOFF "%c   %s\n", part->start,
+	    (DADDR_T) (part->start + part->len - 1), part->len, size, unit,
+	    part->desc);
     }
     else {
 
 	/* Print the layout */
 	printf("%.10" PRIuDADDR "   %.10" PRIuDADDR "   %.10" PRIuDADDR
-	       "   %s\n", part->start, (part->start + part->len - 1),
-	       part->len, part->desc);
+	    "   %s\n", part->start,
+	    (DADDR_T) (part->start + part->len - 1), part->len,
+	    part->desc);
     }
 
     if ((recurse) && (mm->mmtype == MM_DOS) && (part->type == MM_TYPE_VOL)) {
+	// @@@ This assumes 512-byte sectors
 	if (recurse_cnt < 64)
-	    recurse_list[recurse_cnt++] = part->start;
+	    recurse_list[recurse_cnt++] = part->start * 512;
     }
 
     return WALK_CONT;
@@ -122,7 +126,8 @@ static void
 print_header(MM_INFO * mm)
 {
     printf("%s\n", mm->str_type);
-    printf("Sector: %" PRIuDADDR "\n", mm->sect_offset);
+    printf("Offset Sector: %" PRIuDADDR "\n",
+	(DADDR_T) (mm->offset / mm->block_size));
     printf("Units are in %d-byte sectors\n\n", mm->block_size);
     if (print_bytes)
 	printf
@@ -139,7 +144,7 @@ main(int argc, char **argv)
     MM_INFO *mm;
     char *mmtype = NULL;
     int ch;
-    DADDR_T sect_offset = 0;
+    SSIZE_T imgoff = 0;
     uint8_t flags = 0;
     char *imgtype = NULL;
     IMG_INFO *img;
@@ -147,7 +152,6 @@ main(int argc, char **argv)
     progname = argv[0];
 
     while ((ch = getopt(argc, argv, "bi:o:rt:vV")) > 0) {
-	char *cp;
 	switch (ch) {
 	case 'b':
 	    print_bytes = 1;
@@ -156,14 +160,11 @@ main(int argc, char **argv)
 	    imgtype = optarg;
 	    break;
 
-	    // @@@ WE may want this to support the @ syntax...
 	case 'o':
-	    sect_offset = strtoull(optarg, &cp, 0);
-	    if (*cp || cp == optarg) {
-		fprintf(stderr, "Bad sector offset\n");
-		usage();
+	    if ((imgoff = parse_offset(optarg)) == -1) {
+		tsk_error_print(stderr);
+		exit(1);
 	    }
-
 	    break;
 	case 'r':
 	    recurse = 1;
@@ -190,25 +191,31 @@ main(int argc, char **argv)
 	usage();
     }
 
-    /* open the image - do not use an offset */
-    img =
-	img_open(imgtype, "0", argc - optind,
-		 (const char **) &argv[optind]);
+    /* open the image */
+    img = img_open(imgtype, argc - optind, (const char **) &argv[optind]);
+
+    if (img == NULL) {
+	tsk_error_print(stderr);
+	exit(1);
+    }
 
     /* process the partition tables */
-    mm = mm_open(img, mmtype, sect_offset);
+    mm = mm_open(img, (OFF_T) imgoff, mmtype);
     if (mm == NULL) {
-	if (mmtype == NULL)
-	    printf("Error determining partition system type\n");
-	else
-	    fprintf(stderr, "Error processing image\n");
+	tsk_error_print(stderr);
+	if (tsk_errno == TSK_ERR_MM_UNSUPTYPE)
+	    mm_print_types(stderr);
 	exit(1);
     }
 
     print_header(mm);
 
-    mm->part_walk(mm, mm->first_part, mm->last_part, flags,
-		  part_act, NULL);
+    if (mm->part_walk(mm, mm->first_part, mm->last_part, flags,
+	    part_act, NULL)) {
+	tsk_error_print(stderr);
+	mm->close(mm);
+	exit(1);
+    }
 
     mm->close(mm);
     if ((recurse) && (mm->mmtype == MM_DOS)) {
@@ -218,14 +225,20 @@ main(int argc, char **argv)
 	recurse = 0;
 
 	for (i = 0; i < recurse_cnt; i++) {
-	    mm = mm_open(img, NULL, recurse_list[i]);
+	    mm = mm_open(img, recurse_list[i], NULL);
 	    if (mm != NULL) {
 		printf("\n\n");
 		print_header(mm);
-		mm->part_walk(mm, mm->first_part, mm->last_part, flags,
-			      part_act, NULL);
+		if (mm->part_walk(mm, mm->first_part, mm->last_part, flags,
+			part_act, NULL)) {
+		    tsk_errno = 0;
+		}
 		mm->close(mm);
 	    }
+	    else {
+		/* Ignore error in this case and reset */
+		tsk_errno = 0;
+	    }
 	}
     }
 
