@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2005/09/02 23:34:04 $
+ * $Date: 2006/04/06 01:15:57 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2005 Brian Carrier.  All rights reserved
@@ -23,15 +23,14 @@ void
 usage()
 {
     fprintf(stderr,
-	    "%s [-i imgtype] [-o imgoffset] [-vV] [-t mmtype] image [images]\n",
-	    progname);
+	"%s [-i imgtype] [-o imgoffset] [-vV] [-t mmtype] image [images]\n",
+	progname);
     fprintf(stderr, "\t-t mmtype: The type of partition system\n");
     fprintf(stderr, "\t-i imgtype: The format of the image file\n");
     fprintf(stderr,
-	    "\t-o imgoffset: Offset to the start of the volume that contains the partition system (in sectors)\n");
+	"\t-o imgoffset: Offset to the start of the volume that contains the partition system (in sectors)\n");
     fprintf(stderr, "\t-v: verbose output\n");
     fprintf(stderr, "\t-V: print the version\n");
-    fprintf(stderr, "Supported partition systems:\n");
     mm_print_types(stderr);
     fprintf(stderr, "Supported image formats:\n");
     img_print_types(stderr);
@@ -52,25 +51,22 @@ main(int argc, char **argv)
     MM_INFO *mm;
     char *mmtype = NULL;
     int ch;
-    DADDR_T sect_offset = 0;
+    SSIZE_T imgoff = 0;
     char *imgtype = NULL;
     IMG_INFO *img;
 
     progname = argv[0];
 
     while ((ch = getopt(argc, argv, "i:o:t:vV")) > 0) {
-	char *cp;
 	switch (ch) {
 	case 'i':
 	    imgtype = optarg;
 	    break;
 
-	    // @@@ WE may want this to support the @ syntax...
 	case 'o':
-	    sect_offset = strtoull(optarg, &cp, 0);
-	    if (*cp || cp == optarg) {
-		fprintf(stderr, "Bad sector offset\n");
-		usage();
+	    if ((imgoff = parse_offset(optarg)) == -1) {
+		tsk_error_print(stderr);
+		exit(1);
 	    }
 
 	    break;
@@ -96,18 +92,21 @@ main(int argc, char **argv)
 	usage();
     }
 
-    /* open the image - do not use an offset */
-    img =
-	img_open(imgtype, "0", argc - optind,
-		 (const char **) &argv[optind]);
+    /* open the image */
+    if ((img =
+	    img_open(imgtype, argc - optind,
+		(const char **) &argv[optind])) == NULL) {
+	tsk_error_print(stderr);
+	exit(1);
+    }
+
 
     /* process the partition tables */
-    mm = mm_open(img, mmtype, sect_offset);
-    if (mm == NULL) {
-	if (mmtype == NULL)
-	    printf("Error determining partition system type\n");
-	else
-	    fprintf(stderr, "Error processing image\n");
+    if ((mm = mm_open(img, imgoff, mmtype)) == NULL) {
+	tsk_error_print(stderr);
+	if (tsk_errno == TSK_ERR_MM_UNSUPTYPE)
+	    mm_print_types(stderr);
+
 	exit(1);
     }
 
