@@ -1,9 +1,10 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2005/09/02 23:34:04 $
+ * $Date: 2006/04/07 21:21:03 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
+ * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
  * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
  *
  * SUN - Sun VTOC code
@@ -24,6 +25,8 @@ static char *
 sun_get_desc(uint8_t fstype)
 {
     char *str = mymalloc(64);
+    if (str == NULL)
+	return "";
     switch (fstype) {
 
     case 0:
@@ -73,36 +76,47 @@ sun_get_desc(uint8_t fstype)
  */
 
 static uint8_t
-sun_load_table_i386(MM_INFO * mm, sun_dlabel_i386 * dlabel_x86,
-		    uint8_t test)
+sun_load_table_i386(MM_INFO * mm, sun_dlabel_i386 * dlabel_x86)
 {
     uint32_t idx = 0;
+    DADDR_T max_addr = (mm->img_info->size - mm->offset) / mm->block_size;	// max sector
 
     if (verbose)
 	fprintf(stderr, "load_table_i386: Number of partitions: %d\n",
-		getu16(mm, dlabel_x86->num_parts));
+	    getu16(mm, dlabel_x86->num_parts));
 
     /* Cycle through the partitions, there are either 8 or 16 */
     for (idx = 0; idx < getu16(mm, dlabel_x86->num_parts); idx++) {
 
 	if (verbose)
 	    fprintf(stderr,
-		    "load_table_i386: %d  Starting Sector: %lu  Size: %lu  Type: %d\n",
-		    idx,
-		    (ULONG) getu32(mm, dlabel_x86->part[idx].start_sec),
-		    (ULONG) getu32(mm, dlabel_x86->part[idx].size_sec),
-		    getu16(mm, dlabel_x86->part[idx].type));
+		"load_table_i386: %d  Starting Sector: %lu  Size: %lu  Type: %d\n",
+		idx,
+		(ULONG) getu32(mm, dlabel_x86->part[idx].start_sec),
+		(ULONG) getu32(mm, dlabel_x86->part[idx].size_sec),
+		getu16(mm, dlabel_x86->part[idx].type));
 
 	if (getu32(mm, dlabel_x86->part[idx].size_sec) == 0)
 	    continue;
 
+	if (getu32(mm, dlabel_x86->part[idx].start_sec) > max_addr) {
+	    tsk_errno = TSK_ERR_MM_BLK_NUM;
+	    snprintf(tsk_errstr, TSK_ERRSTR_L,
+		"sun_load_i386: Starting sector too large for image");
+	    tsk_errstr2[0] = '\0';
+	    return 1;
+	}
+
+
 	/* Add the partition to the internal sorted list */
-	mm_part_add(mm,
-		    (DADDR_T) getu32(mm, dlabel_x86->part[idx].start_sec),
-		    (DADDR_T) getu32(mm, dlabel_x86->part[idx].size_sec),
-		    MM_TYPE_VOL,
-		    sun_get_desc(getu16(mm, dlabel_x86->part[idx].type)),
-		    -1, idx);
+	if (NULL == mm_part_add(mm,
+		(DADDR_T) getu32(mm, dlabel_x86->part[idx].start_sec),
+		(DADDR_T) getu32(mm, dlabel_x86->part[idx].size_sec),
+		MM_TYPE_VOL,
+		sun_get_desc(getu16(mm, dlabel_x86->part[idx].type)),
+		-1, idx)) {
+	    return 1;
+	}
     }
 
     return 0;
@@ -113,11 +127,11 @@ sun_load_table_i386(MM_INFO * mm, sun_dlabel_i386 * dlabel_x86,
  * sun_load_table
  */
 static uint8_t
-sun_load_table_sparc(MM_INFO * mm, sun_dlabel_sparc * dlabel_sp,
-		     uint8_t test)
+sun_load_table_sparc(MM_INFO * mm, sun_dlabel_sparc * dlabel_sp)
 {
     uint32_t idx = 0;
     uint32_t cyl_conv;
+    DADDR_T max_addr = (mm->img_info->size - mm->offset) / mm->block_size;	// max sector
 
     /* The value to convert cylinders to sectors */
     cyl_conv = getu16(mm, dlabel_sp->sec_per_tr) *
@@ -125,7 +139,7 @@ sun_load_table_sparc(MM_INFO * mm, sun_dlabel_sparc * dlabel_sp,
 
     if (verbose)
 	fprintf(stderr, "load_table_sparc: Number of partitions: %d\n",
-		getu16(mm, dlabel_sp->num_parts));
+	    getu16(mm, dlabel_sp->num_parts));
 
     /* Cycle through the partitions, there are either 8 or 16 */
     for (idx = 0; idx < getu16(mm, dlabel_sp->num_parts); idx++) {
@@ -133,25 +147,33 @@ sun_load_table_sparc(MM_INFO * mm, sun_dlabel_sparc * dlabel_sp,
 	    getu32(mm, dlabel_sp->part_layout[idx].start_cyl);
 
 	uint32_t part_size = getu32(mm,
-				    dlabel_sp->part_layout[idx].size_blk);
+	    dlabel_sp->part_layout[idx].size_blk);
 
 	if (verbose)
 	    fprintf(stderr,
-		    "load_table_sparc: %d  Starting Sector: %lu  Size: %lu  Type: %d\n",
-		    idx,
-		    (ULONG) part_start,
-		    (ULONG) part_size,
-		    getu16(mm, dlabel_sp->part_meta[idx].type));
+		"load_table_sparc: %d  Starting Sector: %lu  Size: %lu  Type: %d\n",
+		idx,
+		(ULONG) part_start,
+		(ULONG) part_size,
+		getu16(mm, dlabel_sp->part_meta[idx].type));
 
 	if (part_size == 0)
 	    continue;
 
+	if (part_start > max_addr) {
+	    tsk_errno = TSK_ERR_MM_BLK_NUM;
+	    snprintf(tsk_errstr, TSK_ERRSTR_L,
+		"sun_load_sparc: Starting sector too large for image");
+	    tsk_errstr2[0] = '\0';
+	    return 1;
+	}
+
+
 	/* Add the partition to the internal sorted list */
-	mm_part_add(mm, (DADDR_T) part_start, (DADDR_T) part_size,
-		    MM_TYPE_VOL,
-		    sun_get_desc(getu16
-				 (mm, dlabel_sp->part_meta[idx].type)), -1,
-		    idx);
+	if (NULL == mm_part_add(mm, (DADDR_T) part_start,
+		(DADDR_T) part_size, MM_TYPE_VOL, sun_get_desc(getu16(mm,
+			dlabel_sp->part_meta[idx].type)), -1, idx))
+	    return 1;
     }
 
     return 0;
@@ -163,9 +185,11 @@ sun_load_table_sparc(MM_INFO * mm, sun_dlabel_sparc * dlabel_sp,
  *
  * This method just finds out if it is sparc or Intel and then
  * calls the appropriate method
+ *
+ * Return 0 on success and 1 on error
  */
 static uint8_t
-sun_load_table(MM_INFO * mm, uint8_t test)
+sun_load_table(MM_INFO * mm)
 {
 /* this will need to change if any of the disk label structures change */
 #define LABEL_BUF_SIZE	512
@@ -173,32 +197,37 @@ sun_load_table(MM_INFO * mm, uint8_t test)
     sun_dlabel_sparc *dlabel_sp;
     sun_dlabel_i386 *dlabel_x86;
     char buf[LABEL_BUF_SIZE];
+    SSIZE_T cnt;
+    DADDR_T taddr = mm->offset / mm->block_size + SUN_SPARC_PART_SOFFSET;
 
 
     /* Sanity check in case label sizes change */
     if ((sizeof(*dlabel_sp) > LABEL_BUF_SIZE) ||
 	(sizeof(*dlabel_x86) > LABEL_BUF_SIZE)) {
-	error("sun_load_table: Buffer smaller than label sizes");
+	tsk_errno = TSK_ERR_MM_BUF;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "sun_load_table: Buffer smaller than label sizes");
+	tsk_errstr2[0] = '\0';
+	return 1;
     }
 
     if (verbose)
 	fprintf(stderr, "sun_load_table: Trying sector: %" PRIuDADDR "\n",
-		mm->sect_offset);
+	    taddr);
 
     /* Try the given offset */
-    if (mm_read_block_nobuf
-	(mm, (char *) &buf, LABEL_BUF_SIZE,
-	 mm->sect_offset) != LABEL_BUF_SIZE) {
-	if (test) {
-	    fprintf(stderr,
-		    "Error reading SUN Disk Label in Sector: %" PRIuDADDR
-		    "\n", mm->sect_offset);
-	    return 1;
-	}
-	else {
-	    error("Error reading SUN Disk Label in Sector: %" PRIuDADDR
-		  "\n", mm->sect_offset);
+    cnt = mm_read_block_nobuf
+	(mm, (char *) &buf, LABEL_BUF_SIZE, SUN_SPARC_PART_SOFFSET);
+
+    /* If -1 is returned, tsk_errno is already set */
+    if (cnt != LABEL_BUF_SIZE) {
+	if (cnt != -1) {
+	    tsk_errno = TSK_ERR_MM_READ;
+	    tsk_errstr[0] = '\0';
 	}
+	snprintf(tsk_errstr2, TSK_ERRSTR_L,
+	    "SUN Disk Label in Sector: %" PRIuDADDR, taddr);
+	return 1;
     }
 
 
@@ -210,92 +239,104 @@ sun_load_table(MM_INFO * mm, uint8_t test)
      * */
     dlabel_sp = (sun_dlabel_sparc *) buf;
     dlabel_x86 = (sun_dlabel_i386 *) buf;
-    if (guessu16(mm, dlabel_sp->magic, SUN_MAGIC) == 0) {
-
+    if (mm_guessu16(mm, dlabel_sp->magic, SUN_MAGIC) == 0) {
 	if (getu32(mm, dlabel_sp->sanity) == SUN_SANITY) {
-	    return sun_load_table_sparc(mm, dlabel_sp, test);
+	    return sun_load_table_sparc(mm, dlabel_sp);
 	}
 	else if (getu32(mm, dlabel_x86->sanity) == SUN_SANITY) {
-	    return sun_load_table_i386(mm, dlabel_x86, test);
+	    return sun_load_table_i386(mm, dlabel_x86);
 	}
     }
 
 
     /* Now try the next sector, which is where the intel 
      * could be stored */
-    mm->sect_offset++;
 
+    taddr = mm->offset / mm->block_size / SUN_I386_PART_SOFFSET;
     if (verbose)
 	fprintf(stderr, "sun_load_table: Trying sector: %" PRIuDADDR "\n",
-		mm->sect_offset);
+	    taddr + 1);
 
-    if (mm_read_block_nobuf
-	(mm, (char *) &buf, LABEL_BUF_SIZE,
-	 mm->sect_offset) != LABEL_BUF_SIZE) {
-	if (test) {
-	    fprintf(stderr,
-		    "Error reading SUN (Intel) Disk Label in Sector: %"
-		    PRIuDADDR "\n", mm->sect_offset);
-	    return 1;
-	}
-	else {
-	    error("Error reading SUN (Intel) Disk Label in Sector: %"
-		  PRIuDADDR "\n", mm->sect_offset);
-	}
+    cnt = mm_read_block_nobuf
+	(mm, (char *) &buf, LABEL_BUF_SIZE, SUN_I386_PART_SOFFSET);
 
+    if (cnt != LABEL_BUF_SIZE) {
+	if (cnt != -1) {
+	    tsk_errno = TSK_ERR_MM_READ;
+	    tsk_errstr[0] = '\0';
+	}
+	snprintf(tsk_errstr2, TSK_ERRSTR_L,
+	    "SUN (Intel) Disk Label in Sector: %" PRIuDADDR, taddr);
+	return 1;
     }
 
     dlabel_x86 = (sun_dlabel_i386 *) buf;
-    if (guessu16(mm, dlabel_x86->magic, SUN_MAGIC)) {
-	if (test)
-	    return 1;
-	else
-	    error("Invalid SUN (intel) partition table (Sector: %"
-		  PRIuDADDR ") %x\n", mm->sect_offset, getu16(mm,
-							      dlabel_sp->
-							      magic));
+    if (mm_guessu16(mm, dlabel_x86->magic, SUN_MAGIC)) {
+	tsk_errno = TSK_ERR_MM_MAGIC;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "SUN (intel) partition table (Sector: %"
+	    PRIuDADDR ") %x", taddr, getu16(mm, dlabel_sp->magic));
+	tsk_errstr2[0] = '\0';
+	return 1;
     }
 
     if (getu32(mm, dlabel_x86->sanity) != SUN_SANITY) {
-	if (test)
-	    return 1;
-	else
-	    error("Invalid SUN (intel) sanity value (Sector: %" PRIuDADDR
-		  ") %x\n", mm->sect_offset, getu16(mm, dlabel_sp->magic));
+	tsk_errno = TSK_ERR_MM_MAGIC;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "SUN (intel) sanity value (Sector: %" PRIuDADDR
+	    ") %x", taddr, getu16(mm, dlabel_sp->magic));
+	tsk_errstr2[0] = '\0';
+	return 1;
     }
 
-    return sun_load_table_i386(mm, dlabel_x86, test);
+    return sun_load_table_i386(mm, dlabel_x86);
 }
 
 
 /* 
  * Walk the partitions that have already been loaded during _open
  *
+ * Return 1 on error and 0 on success
  */
-void
+uint8_t
 sun_part_walk(MM_INFO * mm, PNUM_T start, PNUM_T last, int flags,
-	      MM_PART_WALK_FN action, char *ptr)
+    MM_PART_WALK_FN action, char *ptr)
 {
     MM_PART *part;
     unsigned int cnt = 0;
 
-    if (start < mm->first_part || start > mm->last_part)
-	error("Invalid starting partition: %d", start);
+    if (start < mm->first_part || start > mm->last_part) {
+	tsk_errno = TSK_ERR_MM_WALK_RNG;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "sun_part_walk: Starting partition: %d", start);
+	tsk_errstr2[0] = '\0';
+	return 1;
+    }
 
-    if (last < mm->first_part || last > mm->last_part)
-	error("Invalid ending partition: %d", last);
+    if (last < mm->first_part || last > mm->last_part) {
+	tsk_errno = TSK_ERR_MM_WALK_RNG;
+	snprintf(tsk_errstr, TSK_ERRSTR_L,
+	    "sun_part_walk: Ending partition: %d", last);
+	tsk_errstr2[0] = '\0';
+	return 1;
+    }
 
     part = mm->part_list;
     while ((part != NULL) && (cnt <= last)) {
 
-	if (cnt >= start)
-	    action(mm, cnt, part, 0, ptr);
+	if (cnt >= start) {
+	    int retval = action(mm, cnt, part, 0, ptr);
+	    if (retval == WALK_STOP)
+		return 0;
+	    else if (retval == WALK_ERROR)
+		return 1;
+	}
 
 	part = part->next;
 	cnt++;
     }
 
-    return;
+    return 0;
 }
 
 
@@ -307,16 +348,17 @@ sun_close(MM_INFO * mm)
 }
 
 MM_INFO *
-sun_open(IMG_INFO * img_info, DADDR_T offset, uint8_t test)
+sun_open(IMG_INFO * img_info, DADDR_T offset)
 {
     MM_INFO *mm = (MM_INFO *) mymalloc(sizeof(*mm));
+    if (mm == NULL)
+	return NULL;
 
     mm->img_info = img_info;
     mm->mmtype = MM_SUN;
     mm->str_type = "Sun VTOC";
 
-    /* If an offset was given, then use that too */
-    mm->sect_offset = offset + SUN_SPARC_PART_OFFSET;
+    mm->offset = offset;
 
     /* inititialize settings */
     mm->part_list = NULL;
@@ -330,13 +372,16 @@ sun_open(IMG_INFO * img_info, DADDR_T offset, uint8_t test)
     mm->close = sun_close;
 
     /* Load the partitions into the sorted list */
-    if (sun_load_table(mm, test)) {
+    if (sun_load_table(mm)) {
 	sun_close(mm);
 	return NULL;
     }
 
     /* fill in the sorted list with the 'unknown' values */
-    mm_part_unused(mm);
+    if (mm_part_unused(mm)) {
+	sun_close(mm);
+	return NULL;
+    }
 
     return mm;
 }
